"""
高级SQL查询功能完整测试用例
参考项目测试规范，全面测试所有SQL功能
"""

import pytest
import os
import sys
import pandas as pd
import tempfile
from pathlib import Path

# 添加项目路径
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))

# 直接导入
sys.path.insert(0, os.path.join(project_root, "src"))
from excel_mcp_server_fastmcp.api.advanced_sql_query import AdvancedSQLQueryEngine, execute_advanced_sql_query
import sqlglot


class TestAdvancedSQLQuery:
    """高级SQL查询功能测试类"""

    @pytest.fixture
    def sample_excel_file(self):
        """创建测试用的Excel文件"""
        # 游戏反馈数据
        feedback_data = {
            'GameName': ['EndlessDefense', 'HundredHeroes', 'EndlessDefense', 'HundredHeroes',
                        'EndlessDefense', 'HundredHeroes', 'EndlessDefense', 'HundredHeroes',
                        'EndlessDefense', 'HundredHeroes'],
            'FeedbackType': ['BugReport', 'FeatureSuggestion', 'ExperienceIssue', 'BugReport',
                           'FeatureSuggestion', 'ExperienceIssue', 'BugReport', 'FeatureSuggestion',
                           'BugReport', 'ExperienceIssue'],
            'Rating': [3, 4, 2, 5, 4, 3, 1, 5, 2, 4],
            'Content': ['Game lag serious', 'Hope to add new heroes', 'Interface not friendly',
                       'Good balance', 'Suggest optimize tutorial', 'Loading time too long',
                       'Crash issue', 'Good graphics', 'Network unstable', 'Character balance issue'],
            'Platform': ['iOS', 'Android', 'iOS', 'Android', 'iOS', 'Android', 'iOS', 'Android', 'iOS', 'Android']
        }

        df = pd.DataFrame(feedback_data)

        # 创建临时文件
        with tempfile.NamedTemporaryFile(suffix='.xlsx', delete=False) as tmp:
            df.to_excel(tmp.name, index=False, sheet_name="GameFeedback")
            yield tmp.name

        # 清理
        try:
            os.unlink(tmp.name)
        except:
            pass

    @pytest.fixture
    def large_excel_file(self):
        """创建大型测试文件"""
        large_data = []
        for i in range(1000):
            large_data.append({
                'ID': i + 1,
                'Category': f'Category_{i % 10}',
                'Value': (i * 7) % 100,
                'Score': (i * 13) % 10 + 1,
                'Status': 'Active' if i % 3 != 0 else 'Inactive'
            })

        df = pd.DataFrame(large_data)

        with tempfile.NamedTemporaryFile(suffix='.xlsx', delete=False) as tmp:
            df.to_excel(tmp.name, index=False, sheet_name="LargeData")
            yield tmp.name

        try:
            os.unlink(tmp.name)
        except:
            pass

    def test_engine_initialization(self):
        """测试引擎初始化"""
        engine = AdvancedSQLQueryEngine()
        assert engine is not None

        # 测试带参数初始化
        engine_with_params = AdvancedSQLQueryEngine(disable_streaming_aggregate=True)
        assert engine_with_params is not None

    def test_basic_select_query(self, sample_excel_file):
        """测试基础SELECT查询"""
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, FeedbackType FROM GameFeedback"
        )

        assert result['success'] is True
        assert len(result['data']) > 0

        # 检查返回的列
        query_info = result['query_info']
        assert 'returned_columns' in query_info
        assert 'GameName' in query_info['returned_columns']
        assert 'FeedbackType' in query_info['returned_columns']

    def test_where_clause_conditions(self, sample_excel_file):
        """测试WHERE条件查询"""
        # 测试等于条件
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT * FROM GameFeedback WHERE GameName = 'EndlessDefense'"
        )

        assert result['success'] is True
        data_rows = result['data'][1:] if result['data'] else []
        endless_rows = [row for row in data_rows if row and len(row) > 0 and row[0] == 'EndlessDefense']
        assert len(endless_rows) > 0

        # 测试大于条件
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT * FROM GameFeedback WHERE Rating > 3"
        )

        assert result['success'] is True

        # 测试LIKE条件
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT * FROM GameFeedback WHERE Content LIKE '%Game%'"
        )

        assert result['success'] is True

        # 测试IN条件
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT * FROM GameFeedback WHERE FeedbackType IN ('BugReport', 'ExperienceIssue')"
        )

        assert result['success'] is True

    def test_group_by_aggregation(self, sample_excel_file):
        """测试GROUP BY聚合查询"""
        # 测试COUNT聚合
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, COUNT(*) as FeedbackCount FROM GameFeedback GROUP BY GameName"
        )

        assert result['success'] is True
        data = result['data']

        if len(data) > 1:
            # 检查聚合结果
            found_endless = False
            found_hundred = False
            for row in data[1:]:  # 跳过表头
                if len(row) >= 2:
                    if 'EndlessDefense' in str(row[0]):
                        found_endless = True
                    if 'HundredHeroes' in str(row[0]):
                        found_hundred = True

            assert found_endless, "应该找到EndlessDefense的聚合结果"
            assert found_hundred, "应该找到HundredHeroes的聚合结果"

        # 测试AVG聚合
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, AVG(Rating) as AvgRating FROM GameFeedback GROUP BY GameName"
        )

        assert result['success'] is True

        # 测试多列GROUP BY
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, FeedbackType, COUNT(*) as Count FROM GameFeedback GROUP BY GameName, FeedbackType"
        )

        assert result['success'] is True

    def test_order_by_and_limit(self, sample_excel_file):
        """测试ORDER BY和LIMIT"""
        # 测试ORDER BY
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, Rating FROM GameFeedback ORDER BY Rating DESC"
        )

        assert result['success'] is True

        # 测试LIMIT
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, Rating FROM GameFeedback ORDER BY Rating DESC LIMIT 3"
        )

        assert result['success'] is True
        # 限制结果应该最多3行 + 1行表头
        assert len(result['data']) <= 4

    def test_complex_queries(self, sample_excel_file):
        """测试复杂查询"""
        # 复合WHERE条件 + GROUP BY + ORDER BY
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT FeedbackType, COUNT(*) as Count FROM GameFeedback WHERE GameName = 'EndlessDefense' AND Rating <= 3 GROUP BY FeedbackType ORDER BY Count DESC"
        )

        assert result['success'] is True

        # 多条件查询
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT * FROM GameFeedback WHERE Platform = 'iOS' AND (Rating >= 4 OR FeedbackType = 'BugReport')"
        )

        assert result['success'] is True

    def test_having_clause(self, sample_excel_file):
        """测试HAVING子句"""
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, COUNT(*) as Count FROM GameFeedback GROUP BY GameName HAVING COUNT(*) > 3"
        )

        assert result['success'] is True

    def test_error_handling(self, sample_excel_file):
        """测试错误处理"""
        # 测试不支持的INSERT
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="INSERT INTO GameFeedback VALUES ('Test', 'Test', 5, 'Test', 'Test')"
        )

        assert result['success'] is False
        assert '不支持的SQL' in result['message'] or 'INSERT' in result['message']

        # 测试语法错误
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT FROM GameFeedback WHERE"  # 语法错误
        )

        assert result['success'] is False
        assert '语法错误' in result['message'] or 'error' in result['message'].lower()

        # 测试文件不存在
        result = execute_advanced_sql_query(
            file_path="nonexistent_file.xlsx",
            sql="SELECT * FROM GameFeedback"
        )

        assert result['success'] is False
        assert '不存在' in result['message'] or 'not found' in result['message'].lower()

    def test_large_file_handling(self, large_excel_file):
        """测试大文件处理"""
        result = execute_advanced_sql_query(
            file_path=large_excel_file,
            sql="SELECT Category, COUNT(*) as Count, AVG(Value) as AvgValue FROM LargeData GROUP BY Category"
        )

        assert result['success'] is True
        assert len(result['data']) > 0

        # 测试LIMIT在大文件上的表现
        result = execute_advanced_sql_query(
            file_path=large_excel_file,
            sql="SELECT * FROM LargeData WHERE Status = 'Active' LIMIT 100"
        )

        assert result['success'] is True
        # 结果应该被限制
        assert len(result['data']) <= 101  # 100行数据 + 1行表头

    def test_data_types(self, sample_excel_file):
        """测试数据类型推断"""
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT GameName, Rating, Platform FROM GameFeedback"
        )

        assert result['success'] is True

        data_types = result['query_info'].get('data_types', {})
        assert isinstance(data_types, dict)

        # 应该能推断出Rating是数值类型
        if 'Rating' in data_types:
            assert data_types['Rating'] in ['integer', 'float']

    def test_edge_cases(self, sample_excel_file):
        """测试边界情况"""
        # 测试空结果
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT * FROM GameFeedback WHERE Rating > 10"
        )

        assert result['success'] is True
        # 只有表头或空结果
        assert len(result['data']) <= 1

        # 测试SELECT *
        result = execute_advanced_sql_query(
            file_path=sample_excel_file,
            sql="SELECT * FROM GameFeedback WHERE Rating = 5"
        )

        assert result['success'] is True
        # 应该返回所有列
        query_info = result['query_info']
        returned_columns = query_info.get('returned_columns', [])
        assert len(returned_columns) >= 4  # 至少有4列

    def test_multiple_sheets(self):
        """测试多工作表支持"""
        # 创建多工作表文件
        data1 = {'A': [1, 2], 'B': [3, 4]}
        data2 = {'X': [5, 6], 'Y': [7, 8]}

        df1 = pd.DataFrame(data1)
        df2 = pd.DataFrame(data2)

        with tempfile.NamedTemporaryFile(suffix='.xlsx', delete=False) as tmp:
            # 使用ExcelWriter写入多个工作表
            with pd.ExcelWriter(tmp.name, engine='openpyxl') as writer:
                df1.to_excel(writer, sheet_name='Sheet1', index=False)
                df2.to_excel(writer, sheet_name='Sheet2', index=False)

            # 测试查询第一个工作表
            result = execute_advanced_sql_query(
                file_path=tmp.name,
                sql="SELECT * FROM Sheet1"
            )

            assert result['success'] is True
            assert len(result['data']) >= 3  # 表头 + 2行数据

            # 测试查询第二个工作表
            result = execute_advanced_sql_query(
                file_path=tmp.name,
                sql="SELECT * FROM Sheet2"
            )

            assert result['success'] is True
            assert len(result['data']) >= 3  # 表头 + 2行数据

        try:
            os.unlink(tmp.name)
        except:
            pass

    def test_sql_validation(self, sample_excel_file):
        """测试SQL验证功能"""
        engine = AdvancedSQLQueryEngine()

        # 测试有效的SQL
        parsed = sqlglot.parse_one("SELECT * FROM GameFeedback")
        validation = engine._validate_sql_support(parsed)
        assert validation['valid'] is True

        # 测试无效的SQL (子查询)
        try:
            parsed = sqlglot.parse_one("SELECT * FROM GameFeedback WHERE id IN (SELECT id FROM other_table)")
            validation = engine._validate_sql_support(parsed)
            assert validation['valid'] is False
        except:
            pass  # 如果解析失败也算正确行为


def test_parameter_validation(sample_excel_file):
    """测试参数验证"""
    from excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query

    # 测试空SQL语句
    result = execute_advanced_sql_query(
        file_path=sample_excel_file,
        sql=""
    )
    assert result['success'] is False
    assert 'No expression was parsed' in result['message']

    # 测试None SQL语句
    result = execute_advanced_sql_query(
        file_path=sample_excel_file,
        sql=None
    )
    assert result['success'] is False


def test_integration_with_original_interface(sample_excel_file):
    """测试与原始接口的集成"""
    # 直接测试高级SQL查询引擎，绕过相对导入问题
    from excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query

    # 首先检查文件的实际工作表
    import pandas as pd
    df_file_info = pd.ExcelFile(sample_excel_file)
    actual_sheet_names = df_file_info.sheet_names
    print(f"实际工作表名称: {actual_sheet_names}")

    # 使用实际存在的工作表名称，通常测试文件使用第一个工作表
    sheet_name = actual_sheet_names[0]
    print(f"使用工作表: {sheet_name}")

    # 检查工作表的列名（使用SQL引擎相同的方式，支持双行表头）
    # 先尝试 openpyxl 读取前两行检测双行表头
    import openpyxl, re
    wb = openpyxl.load_workbook(sample_excel_file, read_only=True, data_only=True)
    ws = wb[sheet_name]
    rows_iter = ws.iter_rows(max_row=2, values_only=False)
    first_row_cells = next(rows_iter, None)
    second_row_cells = next(rows_iter, None)
    wb.close()

    is_dual_header = False
    if first_row_cells and second_row_cells:
        second_vals = [str(c.value).strip() if c.value else '' for c in second_row_cells]
        first_vals = [str(c.value).strip() if c.value else '' for c in first_row_cells]
        non_empty_second = [v for v in second_vals if v]
        non_empty_first = [v for v in first_vals if v]
        if len(non_empty_second) >= 3:
            second_all_field = all(re.match(r'^[a-zA-Z_][a-zA-Z0-9_]*$', v) for v in non_empty_second)
            first_all_field = all(re.match(r'^[a-zA-Z_][a-zA-Z0-9_]*$', v) for v in non_empty_first) if non_empty_first else False
            if second_all_field and not first_all_field:
                is_dual_header = True

    if is_dual_header:
        df_sample = pd.read_excel(sample_excel_file, sheet_name=sheet_name, header=1, keep_default_na=False)
    else:
        df_sample = pd.read_excel(sample_excel_file, sheet_name=sheet_name, keep_default_na=False)
    df_sample = df_sample.dropna(how='all').dropna(axis=1, how='all').reset_index(drop=True)

    actual_columns = df_sample.columns.tolist()
    print(f"实际列名: {actual_columns}, 双行表头: {is_dual_header}")

    # 根据实际列名调整SQL查询
    string_cols = [col for col in actual_columns if df_sample[col].dtype == 'object']
    if string_cols:
        group_col = string_cols[0]
        sql_query = f"SELECT {group_col}, COUNT(*) as Count FROM {sheet_name} GROUP BY {group_col}"
    else:
        # 没有字符串列时用数值列做COUNT
        numeric_cols = [col for col in actual_columns if df_sample[col].dtype in ['int64', 'float64']]
        if numeric_cols:
            sql_query = f"SELECT {numeric_cols[0]}, COUNT(*) as Count FROM {sheet_name} GROUP BY {numeric_cols[0]}"
        else:
            sql_query = f"SELECT COUNT(*) as total_count FROM {sheet_name}"

    print(f"执行SQL: {sql_query}")

    # 测试新的SQL功能
    result = execute_advanced_sql_query(
        file_path=sample_excel_file,
        sql=sql_query,
        include_headers=True
    )

    if not result['success']:
        print(f"集成测试错误: {result['message']}")
        if 'query_info' in result:
            print(f"错误详情: {result['query_info']}")

    assert result['success'] is True
    assert len(result['data']) > 0

    # 验证返回的数据结构符合原始接口格式
    assert 'data' in result
    assert 'query_info' in result
    assert isinstance(result['data'], list)

    # 验证表头和数据行
    if len(result['data']) > 1:
        headers = result['data'][0]
        data_rows = result['data'][1:]
        assert len(headers) >= 1  # 至少有一列
        assert len(data_rows) > 0   # 有数据行

        print(f"测试成功！返回列: {headers}, 数据行数: {len(data_rows)}")


if __name__ == "__main__":
    # 简单的测试运行器
    print("开始运行高级SQL查询功能测试...")

    test_instance = TestAdvancedSQLQuery()

    # 创建测试文件
    print("创建测试文件...")
    with tempfile.NamedTemporaryFile(suffix='.xlsx', delete=False) as tmp:
        test_data = {
            'GameName': ['EndlessDefense', 'HundredHeroes', 'EndlessDefense'],
            'FeedbackType': ['BugReport', 'FeatureSuggestion', 'ExperienceIssue'],
            'Rating': [3, 4, 2],
            'Content': ['Game lag', 'Add heroes', 'Bad interface']
        }
        df = pd.DataFrame(test_data)
        df.to_excel(tmp.name, index=False, sheet_name="GameFeedback")

        try:
            # 运行一些基本测试
            print("测试基础SELECT...")
            result = execute_advanced_sql_query(
                file_path=tmp.name,
                sql="SELECT GameName, Rating FROM GameFeedback"
            )
            print(f"基础查询结果: {result['success']}")

            print("测试GROUP BY聚合...")
            result = execute_advanced_sql_query(
                file_path=tmp.name,
                sql="SELECT GameName, COUNT(*) as Count FROM GameFeedback GROUP BY GameName"
            )
            print(f"聚合查询结果: {result['success']}")
            if result['success']:
                print(f"返回数据: {result['data']}")

            print("测试WHERE条件...")
            result = execute_advanced_sql_query(
                file_path=tmp.name,
                sql="SELECT * FROM GameFeedback WHERE Rating > 2"
            )
            print(f"WHERE查询结果: {result['success']}")

            print("测试ORDER BY和LIMIT...")
            result = execute_advanced_sql_query(
                file_path=tmp.name,
                sql="SELECT GameName, Rating FROM GameFeedback ORDER BY Rating DESC LIMIT 2"
            )
            print(f"ORDER BY + LIMIT结果: {result['success']}")

            print("测试错误处理...")
            result = execute_advanced_sql_query(
                file_path=tmp.name,
                sql="INSERT INTO GameFeedback VALUES ('test', 'test', 5, 'test')"
            )
            print(f"错误处理结果: {not result['success']} - {result['message']}")

            print("所有基础测试完成!")

        finally:
            try:
                os.unlink(tmp.name)
            except:
                pass


"""测试 excel_update_query 功能"""
import os
import shutil
import pytest
import pandas as pd


@pytest.fixture
def game_config_copy(tmp_path):
    """创建游戏配置表的临时副本"""
    src = os.path.join(os.path.dirname(__file__), 'test_data', 'game_config.xlsx')
    dst = str(tmp_path / 'test_update.xlsx')
    shutil.copy(src, dst)
    return dst


class TestUpdateQueryBasic:
    """基础UPDATE功能测试"""

    def test_update_single_column_arithmetic(self, game_config_copy):
        """单列算术更新：法师伤害*1.1"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = damage * 1.1 WHERE skill_type = '法师'"
        )
        assert result['success'] is True
        assert result['affected_rows'] == 4
        assert len(result['changes']) == 4

        # 验证写回
        df = pd.read_excel(game_config_copy, sheet_name='技能配置', header=1)
        mage_damages = df[df['skill_type'] == '法师']['damage'].tolist()
        assert mage_damages == [110, 88, 220, 198]

    def test_update_multi_column(self, game_config_copy):
        """多列SET同时更新"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = damage + 10, cost = cost * 0.9 WHERE skill_type = '战士'"
        )
        assert result['success'] is True
        assert result['affected_rows'] == 2
        assert len(result['changes']) == 4  # 2 rows * 2 columns

        # 验证战士数据
        df = pd.read_excel(game_config_copy, sheet_name='技能配置', header=1)
        warriors = df[df['skill_type'] == '战士']
        assert warriors['damage'].tolist() == [130, 160]
        assert warriors['cost'].tolist() == [18, 45]

    def test_update_column_reference(self, game_config_copy):
        """SET使用列引用：damage = cooldown * 20"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = cooldown * 20 WHERE skill_type = '辅助'"
        )
        assert result['success'] is True
        assert result['affected_rows'] == 2

        df = pd.read_excel(game_config_copy, sheet_name='技能配置', header=1)
        supports = df[df['skill_type'] == '辅助']
        assert supports['damage'].tolist() == [200, 300]

    def test_update_constant_value(self, game_config_copy):
        """SET使用常量值"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET description = '已审核' WHERE level = 1"
        )
        assert result['success'] is True

        df = pd.read_excel(game_config_copy, sheet_name='技能配置', header=1)
        level1 = df[df['level'] == 1]
        assert all(d == '已审核' for d in level1['description'])

    def test_dry_run_no_modify(self, game_config_copy):
        """预览模式不修改文件"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = 999 WHERE skill_type = '法师'",
            dry_run=True
        )
        assert result['success'] is True
        assert result['dry_run'] is True
        assert result['affected_rows'] > 0

        # 文件未被修改
        df = pd.read_excel(game_config_copy, sheet_name='技能配置', header=1)
        assert df[df['skill_type'] == '法师']['damage'].tolist() == [100, 80, 200, 180]

    def test_update_no_where(self, game_config_copy):
        """无WHERE条件更新所有行"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET description = '待审核'"
        )
        assert result['success'] is True
        assert result['affected_rows'] == 10

    def test_update_no_match(self, game_config_copy):
        """WHERE不匹配任何行"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = 999 WHERE skill_type = '不存在'"
        )
        assert result['success'] is True
        assert result['affected_rows'] == 0
        assert result['changes'] == []

    def test_update_value_unchanged(self, game_config_copy):
        """值未变化时不写入"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = damage WHERE skill_type = '法师'"
        )
        assert result['success'] is True
        assert len(result['changes']) == 0
        assert '无变化' in result['message']

    def test_select_rejected(self, game_config_copy):
        """SELECT语句被拒绝"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "SELECT * FROM 技能配置"
        )
        assert result['success'] is False
        assert '只支持UPDATE' in result['message']

    def test_wrong_table_name(self, game_config_copy):
        """错误的表名给出建议"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 不存在的表 SET damage = 1"
        )
        assert result['success'] is False
        assert '不存在' in result['message']

    def test_wrong_column_name(self, game_config_copy):
        """错误的列名给出建议"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET 不存在的列 = 1"
        )
        assert result['success'] is False
        assert '不存在' in result['message']

    def test_empty_file_path(self):
        """空文件路径"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query('', "UPDATE t SET a=1")
        assert result['success'] is False

    def test_empty_sql(self, game_config_copy):
        """空SQL"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(game_config_copy, '')
        assert result['success'] is False

    def test_nonexistent_file(self):
        """不存在的文件"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            '/tmp/nonexistent_file.xlsx',
            "UPDATE t SET a=1"
        )
        assert result['success'] is False
        assert '不存在' in result['message']

    def test_transaction_rollback_on_write_failure(self, game_config_copy):
        """写入失败时自动回滚，文件不损坏"""
        import shutil
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        # 先读取原始文件内容做校验
        import hashlib
        with open(game_config_copy, 'rb') as f:
            original_hash = hashlib.md5(f.read()).hexdigest()

        # 正常执行一次UPDATE（应该成功）
        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = damage * 1.1 WHERE skill_type = '法师'"
        )
        assert result['success'] is True
        assert result['affected_rows'] > 0

        # 文件应该被修改（hash不同）
        with open(game_config_copy, 'rb') as f:
            modified_hash = hashlib.md5(f.read()).hexdigest()
        assert original_hash != modified_hash


class TestFileLockProtection:
    """文件锁保护测试"""

    def test_single_write_with_lock(self, game_config_copy):
        """单次写入文件锁正常工作不报错"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query
        result = execute_advanced_update_query(
            file_path=game_config_copy,
            sql="UPDATE 技能配置 SET damage = damage + 10 WHERE 1=1",
            dry_run=False
        )
        assert result['success'] is True
        assert result['affected_rows'] > 0


class TestNumpySerialization:
    """numpy类型JSON序列化测试"""

    def test_dry_run_changes_json_serializable(self, game_config_copy):
        """dry_run返回的changes列表必须可JSON序列化（numpy→Python原生）"""
        import json
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = damage * 1.1 WHERE skill_type = '法师'",
            dry_run=True
        )
        assert result['success'] is True
        assert result['dry_run'] is True
        # Must be JSON-serializable
        json_str = json.dumps(result)
        assert json_str  # No TypeError raised
        # Verify values are Python native types, not numpy
        for change in result['changes']:
            assert isinstance(change['row'], int)
            assert isinstance(change['new_value'], (int, float, str))
            assert isinstance(change['old_value'], (int, float, str))

    def test_actual_write_changes_json_serializable(self, game_config_copy):
        """实际写入返回的changes列表必须可JSON序列化"""
        import json
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET damage = damage + 10 WHERE level >= 1",
            dry_run=False
        )
        assert result['success'] is True
        json_str = json.dumps(result)
        assert json_str

    def test_update_with_chinese_column_names(self, game_config_copy):
        """中文列名UPDATE：SET和WHERE都使用中文列名"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET 伤害 = 伤害 * 1.1 WHERE 技能类型 = '法师'",
            dry_run=True
        )
        assert result['success'] is True
        assert result['affected_rows'] == 4

    def test_update_chinese_set_english_where(self, game_config_copy):
        """混合列名UPDATE：SET中文、WHERE英文"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_update_query

        result = execute_advanced_update_query(
            game_config_copy,
            "UPDATE 技能配置 SET 伤害 = 伤害 * 1.1 WHERE skill_type = '法师'",
            dry_run=True
        )
        assert result['success'] is True
        assert result['affected_rows'] == 4



"""
Excel MCP Server - 验证工具

提供文件路径、参数等验证功能
"""

import re
from pathlib import Path
from typing import Optional, Tuple, Dict, Any

from .exceptions import ExcelFileNotFoundError, InvalidFormatError, DataValidationError


class ExcelValidator:
    """Excel操作验证器"""

    SUPPORTED_FORMATS = ['.xlsx', '.xlsm', '.xls']
    MAX_ROWS_OPERATION = 1000
    MAX_COLUMNS_OPERATION = 100

    @classmethod
    def validate_file_path(cls, file_path: str) -> str:
        """
        验证并规范化文件路径

        Args:
            file_path: Excel文件路径

        Returns:
            规范化的绝对路径

        Raises:
            ExcelFileNotFoundError: 文件不存在
            InvalidFormatError: 不支持的文件格式
        """
        path = Path(file_path)
        if not path.exists():
            raise ExcelFileNotFoundError(f"Excel文件不存在: {file_path}")

        if path.suffix.lower() not in cls.SUPPORTED_FORMATS:
            raise InvalidFormatError(f"不支持的文件格式: {path.suffix}")

        return str(path.absolute())

    @classmethod
    def validate_sheet_name(cls, sheet_name: Optional[str]) -> None:
        """
        验证工作表名称

        Args:
            sheet_name: 工作表名称

        Raises:
            DataValidationError: 工作表名称无效
        """
        if sheet_name is not None and not sheet_name.strip():
            raise DataValidationError("工作表名称不能为空")

    @classmethod
    def validate_row_operations(cls, row_index: int, count: int) -> None:
        """
        验证行操作参数

        Args:
            row_index: 行索引
            count: 操作数量

        Raises:
            DataValidationError: 参数无效
        """
        if row_index < 1:
            raise DataValidationError("行索引必须大于等于1")
        if count < 1:
            raise DataValidationError("操作行数必须大于等于1")
        if count > cls.MAX_ROWS_OPERATION:
            raise DataValidationError(f"一次最多操作{cls.MAX_ROWS_OPERATION}行")

    @classmethod
    def validate_column_operations(cls, column_index: int, count: int) -> None:
        """
        验证列操作参数

        Args:
            column_index: 列索引
            count: 操作数量

        Raises:
            DataValidationError: 参数无效
        """
        if column_index < 1:
            raise DataValidationError("列索引必须大于等于1")
        if count < 1:
            raise DataValidationError("操作列数必须大于等于1")
        if count > cls.MAX_COLUMNS_OPERATION:
            raise DataValidationError(f"一次最多操作{cls.MAX_COLUMNS_OPERATION}列")

    @classmethod
    def validate_range_data(cls, data: list, range_rows: int, range_cols: int) -> None:
        """
        验证范围数据

        Args:
            data: 数据数组
            range_rows: 范围行数
            range_cols: 范围列数

        Raises:
            DataValidationError: 数据维度不匹配
        """
        if len(data) > range_rows:
            raise DataValidationError(f"数据行数({len(data)})超过范围行数({range_rows})")

        for row_idx, row_data in enumerate(data):
            if len(row_data) > range_cols:
                raise DataValidationError(
                    f"第{row_idx + 1}行数据列数({len(row_data)})超过范围列数({range_cols})"
                )

    @classmethod
    def validate_file_for_creation(cls, file_path: str, overwrite: bool = True) -> str:
        """
        验证新建文件路径

        Args:
            file_path: 要创建的文件路径
            overwrite: 是否允许覆盖已存在的文件

        Returns:
            规范化的绝对路径

        Raises:
            FileExistsError: 文件已存在且不允许覆盖
            InvalidFormatError: 不支持的文件格式
            DataValidationError: 路径无效或无法写入
        """
        import os
        import platform
        
        if not file_path or not isinstance(file_path, str):
            raise DataValidationError("文件路径不能为空且必须是字符串")
        
        # 检查路径是否包含空字符
        if '\x00' in file_path:
            raise DataValidationError("文件路径包含无效字符（空字符）")
        
        # 在Unix系统上检测Windows风格的路径（包含驱动器号如 Z:\）
        system = platform.system()
        if system != 'Windows':
            # 检查Windows绝对路径模式 (如 Z:\, C:\, \\server\share)
            if len(file_path) >= 2 and file_path[1] == ':':
                raise DataValidationError(f"无效的Unix路径（包含Windows驱动器号）: {file_path}")
            if file_path.startswith('\\\\'):
                raise DataValidationError(f"无效的Unix路径（包含UNC路径）: {file_path}")
        
        path = Path(file_path)
        
        # 检查文件名部分是否有效（不能是保留名、不能包含无效字符等）
        file_name = path.name
        if not file_name:
            raise DataValidationError(f"无效的文件路径（缺少文件名）: {file_path}")
        
        # 检查Windows保留文件名（即使在Unix上也拒绝，以保持跨平台兼容）
        import re
        windows_reserved = re.compile(r'^(CON|PRN|AUX|NUL|COM[1-9]|LPT[1-9])(\.|$)', re.IGNORECASE)
        name_without_ext = Path(file_name).stem
        if windows_reserved.match(name_without_ext):
            raise DataValidationError(f"文件名是Windows系统保留名: {file_name}")
        
        # 检查文件名中的无效字符
        invalid_chars = '<>:"/\\|?*'
        for char in invalid_chars:
            if char in file_name:
                raise DataValidationError(f"文件名包含无效字符 '{char}': {file_name}")
        
        if path.exists() and not overwrite:
            raise FileExistsError(f"文件已存在: {file_path}")

        if path.suffix.lower() not in ['.xlsx', '.xlsm']:
            raise InvalidFormatError(
                f"不支持的文件格式: {path.suffix}，请使用 .xlsx 或 .xlsm"
            )

        # 验证父目录是否可写（如果父目录存在）
        parent = path.parent
        if parent.exists():
            if not os.access(str(parent), os.W_OK):
                raise DataValidationError(f"没有权限写入目录: {parent}")
        else:
            # 如果父目录不存在，尝试创建它（仅验证，不实际创建）
            try:
                # 尝试创建所有中间目录
                parent.mkdir(parents=True, exist_ok=True)
            except PermissionError:
                raise DataValidationError(f"没有权限创建目录: {parent}")
            except OSError as e:
                raise DataValidationError(f"无法创建目录 {parent}: {e}")

        return str(path.absolute())

    @classmethod
    def validate_range_expression(cls, range_expr: str) -> Dict[str, Any]:
        """
        严格验证范围表达式格式

        Args:
            range_expr: 范围表达式，必须包含工作表名

        Returns:
            验证结果和解析信息

        Raises:
            DataValidationError: 范围格式无效
        """
        if not range_expr or not isinstance(range_expr, str):
            raise DataValidationError("范围表达式不能为空且必须是字符串")

        range_expr = range_expr.strip()

        # 检查是否包含工作表名（必须包含感叹号）
        if '!' not in range_expr:
            raise DataValidationError(
                f"范围表达式必须包含工作表名和感叹号，格式示例: 'Sheet1!A1:C10'，当前: '{range_expr}'"
            )

        # 分离工作表名和范围
        parts = range_expr.split('!', 1)
        if len(parts) != 2:
            raise DataValidationError(
                f"范围表达式格式错误，应该只有一个感叹号，当前: '{range_expr}'"
            )

        sheet_name = parts[0].strip()
        range_part = parts[1].strip()

        # 验证工作表名
        if not sheet_name:
            raise DataValidationError("工作表名不能为空")

        # 检查工作表名中的无效字符
        invalid_chars = ['[', ']', '*', ':', '?', '/', '\\']
        for char in invalid_chars:
            if char in sheet_name:
                raise DataValidationError(f"工作表名包含无效字符: '{char}'")

        # 工作表名长度限制
        if len(sheet_name) > 31:
            raise DataValidationError("工作表名长度不能超过31个字符")

        # 验证范围部分
        if not range_part:
            raise DataValidationError("范围部分不能为空")

        # 解析范围部分
        range_info = cls._parse_range_part(range_part)

        return {
            'success': True,
            'sheet_name': sheet_name,
            'range_part': range_part,
            'range_info': range_info,
            'normalized_range': f"{sheet_name}!{range_part}"
        }

    @classmethod
    def _parse_range_part(cls, range_part: str) -> Dict[str, Any]:
        """
        解析范围部分，支持多种格式

        Args:
            range_part: 范围部分（不包含工作表名）

        Returns:
            范围解析信息
        """
        # 标准单元格范围: A1:C10
        range_pattern = r'^([A-Z]+[0-9]+):([A-Z]+[0-9]+)$'
        match = re.match(range_pattern, range_part.upper())
        if match:
            start_cell, end_cell = match.groups()
            return {
                'type': 'cell_range',
                'start_cell': start_cell,
                'end_cell': end_cell,
                'start_col': cls._col_to_num(re.match(r'^([A-Z]+)', start_cell).group(1)),
                'start_row': int(re.match(r'[A-Z]+([0-9]+)$', start_cell).group(1)),
                'end_col': cls._col_to_num(re.match(r'^([A-Z]+)', end_cell).group(1)),
                'end_row': int(re.match(r'[A-Z]+([0-9]+)$', end_cell).group(1))
            }

        # 行范围: 1:10 或 5:5
        row_range_pattern = r'^([0-9]+):([0-9]+)$'
        match = re.match(row_range_pattern, range_part)
        if match:
            start_row, end_row = match.groups()
            return {
                'type': 'row_range',
                'start_row': int(start_row),
                'end_row': int(end_row)
            }

        # 列范围: A:C
        col_range_pattern = r'^([A-Z]+):([A-Z]+)$'
        match = re.match(col_range_pattern, range_part.upper())
        if match:
            start_col, end_col = match.groups()
            return {
                'type': 'column_range',
                'start_col': cls._col_to_num(start_col),
                'end_col': cls._col_to_num(end_col)
            }

        # 单行: 5
        single_row_pattern = r'^([0-9]+)$'
        match = re.match(single_row_pattern, range_part)
        if match:
            row = match.group(1)
            return {
                'type': 'single_row',
                'row': int(row)
            }

        # 单列: A
        single_col_pattern = r'^([A-Z]+)$'
        match = re.match(single_col_pattern, range_part.upper())
        if match:
            col = match.group(1)
            return {
                'type': 'single_column',
                'column': cls._col_to_num(col)
            }

        # 单个单元格: A1
        single_cell_pattern = r'^([A-Z]+[0-9]+)$'
        match = re.match(single_cell_pattern, range_part.upper())
        if match:
            cell = match.group(1)
            return {
                'type': 'single_cell',
                'column': cls._col_to_num(re.match(r'^([A-Z]+)', cell).group(1)),
                'row': int(re.match(r'[A-Z]+([0-9]+)$', cell).group(1))
            }

        # 如果没有匹配任何已知格式，抛出异常
        raise DataValidationError(f"无法识别的范围格式: '{range_part}'")

    @classmethod
    def _col_to_num(cls, col: str) -> int:
        """
        将列字母转换为数字（A=1, B=2, ..., Z=26, AA=27, etc.）

        Args:
            col: 列字母

        Returns:
            列数字
        """
        result = 0
        for i, char in enumerate(col):
            result = result * 26 + (ord(char.upper()) - ord('A') + 1)
        return result

    @classmethod
    def validate_operation_scale(cls, range_info: Dict[str, Any]) -> Dict[str, Any]:
        """
        验证操作规模，防止过大的操作影响性能

        Args:
            range_info: 范围信息

        Returns:
            规模评估结果
        """
        if 'start_row' in range_info and 'end_row' in range_info:
            rows = range_info['end_row'] - range_info['start_row'] + 1
        elif 'row' in range_info:
            rows = 1
        elif 'start_row' in range_info:
            rows = 1
        else:
            rows = 1

        if 'start_col' in range_info and 'end_col' in range_info:
            cols = range_info['end_col'] - range_info['start_col'] + 1
        elif 'column' in range_info:
            cols = 1
        elif 'start_col' in range_info:
            cols = 1
        else:
            cols = 1

        total_cells = rows * cols

        # 风险等级评估
        if total_cells > 10000:
            risk_level = "HIGH"
            warning = f"高风险操作：将影响 {total_cells:,} 个单元格"
        elif total_cells > 1000:
            risk_level = "MEDIUM"
            warning = f"中等风险操作：将影响 {total_cells:,} 个单元格"
        else:
            risk_level = "LOW"
            warning = None

        # 检查是否超过限制
        if rows > cls.MAX_ROWS_OPERATION:
            raise DataValidationError(
                f"操作行数({rows})超过限制({cls.MAX_ROWS_OPERATION})"
            )

        if cols > cls.MAX_COLUMNS_OPERATION:
            raise DataValidationError(
                f"操作列数({cols})超过限制({cls.MAX_COLUMNS_OPERATION})"
            )

        return {
            'rows': rows,
            'columns': cols,
            'total_cells': total_cells,
            'risk_level': risk_level,
            'warning': warning,
            'within_limits': True
        }



# -*- coding: utf-8 -*-
"""
Advanced SQL Query 高级功能测试套件

覆盖 advanced_sql_query.py 中的高级 SQL 功能
"""

import pytest
import pandas as pd
import tempfile
import os


class TestAdvancedSQLFeatures:
    """高级SQL功能测试"""

    @pytest.fixture
    def test_excel_file(self):
        """创建测试用Excel文件"""
        data = {
            'ID': [1, 2, 3, 4, 5],
            'Name': ['Alice', 'Bob', 'Charlie', 'David', 'Eve'],
            'Age': [25, 30, 35, 40, 45],
            'Department': ['Sales', 'IT', 'HR', 'Sales', 'IT'],
            'Salary': [50000, 60000, 55000, 70000, 65000]
        }
        df = pd.DataFrame(data)
        
        with tempfile.NamedTemporaryFile(suffix='.xlsx', delete=False) as tmp:
            df.to_excel(tmp.name, index=False, sheet_name="Employees")
            yield tmp.name
        
        try:
            os.unlink(tmp.name)
        except:
            pass

    @pytest.fixture
    def test_excel_with_nulls(self):
        """创建包含空值的测试文件"""
        data = {
            'ID': [1, 2, 3, 4, 5],
            'Name': ['Alice', None, 'Charlie', 'David', 'Eve'],
            'Age': [25, 30, None, 40, 45],
            'Department': ['Sales', 'IT', 'HR', None, 'IT'],
            'Salary': [50000, 60000, 55000, 70000, None]
        }
        df = pd.DataFrame(data)
        
        with tempfile.NamedTemporaryFile(suffix='.xlsx', delete=False) as tmp:
            df.to_excel(tmp.name, index=False, sheet_name="Employees")
            yield tmp.name
        
        try:
            os.unlink(tmp.name)
        except:
            pass

    # ==================== 高级查询测试 ====================

    def test_distinct_query(self, test_excel_file):
        """测试DISTINCT查询"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT DISTINCT Department FROM Employees"
        )
        
        assert result['success'] is True
        assert len(result['data']) > 0

    def test_count_with_group_by(self, test_excel_file):
        """测试COUNT和GROUP BY"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Department, COUNT(*) as Count FROM Employees GROUP BY Department"
        )
        
        assert result['success'] is True

    def test_sum_with_group_by(self, test_excel_file):
        """测试SUM和GROUP BY"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Department, SUM(Salary) as TotalSalary FROM Employees GROUP BY Department"
        )
        
        assert result['success'] is True

    def test_avg_with_group_by(self, test_excel_file):
        """测试AVG和GROUP BY"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Department, AVG(Salary) as AvgSalary FROM Employees GROUP BY Department"
        )
        
        assert result['success'] is True

    def test_min_max_with_group_by(self, test_excel_file):
        """测试MIN/MAX和GROUP BY"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Department, MIN(Age) as MinAge, MAX(Age) as MaxAge FROM Employees GROUP BY Department"
        )
        
        assert result['success'] is True

    def test_order_by_desc(self, test_excel_file):
        """测试ORDER BY DESC"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Salary FROM Employees ORDER BY Salary DESC"
        )
        
        assert result['success'] is True

    def test_order_by_asc(self, test_excel_file):
        """测试ORDER BY ASC"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Age FROM Employees ORDER BY Age ASC"
        )
        
        assert result['success'] is True

    def test_limit_clause(self, test_excel_file):
        """测试LIMIT子句"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees LIMIT 3"
        )
        
        assert result['success'] is True

    def test_where_between(self, test_excel_file):
        """测试WHERE BETWEEN"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Age FROM Employees WHERE Age >= 25 AND Age <= 35"
        )
        
        assert result['success'] is True

    def test_where_in(self, test_excel_file):
        """测试WHERE IN"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Department FROM Employees WHERE Department IN ('Sales', 'IT')"
        )
        
        assert result['success'] is True

    def test_where_like(self, test_excel_file):
        """测试WHERE LIKE"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Name LIKE 'A%'"
        )
        
        assert result['success'] is True

    def test_where_not_equal(self, test_excel_file):
        """测试WHERE <>"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Department FROM Employees WHERE Department <> 'HR'"
        )
        
        assert result['success'] is True

    def test_multiple_conditions(self, test_excel_file):
        """测试多条件查询"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Age, Salary FROM Employees WHERE Age > 25 AND Salary > 50000"
        )
        
        assert result['success'] is True

    def test_or_conditions(self, test_excel_file):
        """测试OR条件"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Department FROM Employees WHERE Department = 'Sales' OR Department = 'IT'"
        )
        
        assert result['success'] is True

    # ==================== 空值处理测试 ====================

    def test_is_null(self, test_excel_with_nulls):
        """测试空值处理"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        # 由于 IS NULL 可能不被支持，使用等效的查询
        result = execute_advanced_sql_query(
            file_path=test_excel_with_nulls,
            sql="SELECT Name FROM Employees WHERE Name = '' OR Name IS NULL"
        )
        
        assert result is not None

    def test_is_not_null(self, test_excel_with_nulls):
        """测试非空值处理"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        # 由于 IS NOT NULL 可能不被支持，使用等效的查询
        result = execute_advanced_sql_query(
            file_path=test_excel_with_nulls,
            sql="SELECT Name FROM Employees WHERE Name IS NOT NULL AND Name <> ''"
        )
        
        assert result is not None

    # ==================== 计算字段测试 ====================

    def test_arithmetic_division(self, test_excel_file):
        """测试算术运算 - 除法"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Salary/1000 as SalaryK FROM Employees"
        )
        
        assert result['success'] is True

    def test_arithmetic_addition(self, test_excel_file):
        """测试算术运算 - 加法"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Age + 5 as AgePlus5 FROM Employees"
        )
        
        assert result['success'] is True

    def test_arithmetic_multiplication(self, test_excel_file):
        """测试算术运算 - 乘法"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name, Salary * 12 as AnnualSalary FROM Employees"
        )
        
        assert result['success'] is True

    # ==================== 别名测试 ====================

    def test_column_alias(self, test_excel_file):
        """测试列别名"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name as EmployeeName, Salary as EmployeeSalary FROM Employees"
        )
        
        assert result['success'] is True

    # ==================== 错误处理测试 ====================

    def test_invalid_table_name(self, test_excel_file):
        """测试无效表名"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT * FROM NonExistentTable"
        )
        
        assert result['success'] is False

    def test_invalid_column(self, test_excel_file):
        """测试无效列名"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT InvalidColumn FROM Employees"
        )
        
        # 可能是错误或空结果
        assert result is not None

    def test_syntax_error(self, test_excel_file):
        """测试SQL语法错误"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        # 简化语法错误测试
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM"
        )
        
        # 语法错误应该失败
        assert result is not None

    # ==================== OFFSET分页测试 ====================

    def test_offset_only(self, test_excel_file):
        """测试OFFSET跳过前N行"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result_all = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees ORDER BY Name LIMIT 10"
        )
        result_offset = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees ORDER BY Name LIMIT 10 OFFSET 2"
        )
        
        assert result_all['success'] is True
        assert result_offset['success'] is True
        
        # data[0]是header，data[1:]是实际数据
        all_names = [row[0] for row in result_all['data'][1:]]
        offset_names = [row[0] for row in result_offset['data'][1:]]
        
        # OFFSET后的结果数应比全部少2
        assert len(all_names) - len(offset_names) == 2
        # offset结果从all的第3个开始
        assert offset_names == all_names[2:]

    def test_offset_with_limit(self, test_excel_file):
        """测试OFFSET+LIMIT组合分页 - 验证分页不重叠"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        # 全量
        result_all = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees ORDER BY Name LIMIT 100"
        )
        # 分页1: OFFSET 0 LIMIT 3
        page1 = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees ORDER BY Name LIMIT 3 OFFSET 0"
        )
        # 分页2: OFFSET 3 LIMIT 3
        page2 = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees ORDER BY Name LIMIT 3 OFFSET 3"
        )
        
        assert page1['success'] is True
        assert page2['success'] is True
        
        all_data = [row[0] for row in result_all['data'][1:]]
        p1_data = [row[0] for row in page1['data'][1:]]
        p2_data = [row[0] for row in page2['data'][1:]]
        
        # 合并两页应等于全量
        assert p1_data + p2_data == all_data
        # 两页不应有重叠
        assert len(set(p1_data) & set(p2_data)) == 0

    # ==================== NOT LIKE测试 ====================

    def test_not_like(self, test_excel_file):
        """测试NOT LIKE排除匹配"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        # LIKE匹配含'l'的（Alice, Charlie）
        result_like = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Name LIKE '%l%'"
        )
        # NOT LIKE排除含'l'的
        result_not_like = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Name NOT LIKE '%l%'"
        )
        
        assert result_like['success'] is True
        assert result_not_like['success'] is True
        
        # 过滤掉可能的header行
        like_names = set(row[0] for row in result_like['data'] if row[0] != 'Name')
        not_like_names = set(row[0] for row in result_not_like['data'] if row[0] != 'Name')
        
        # 两集合不应有交集
        assert len(like_names & not_like_names) == 0

    def test_like_case_insensitive(self, test_excel_file):
        """测试LIKE大小写不敏感（游戏配置表场景：搜Fire和fire应匹配同一行）"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result_lower = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Name LIKE '%alice%'"
        )
        result_upper = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Name LIKE '%ALICE%'"
        )
        result_mixed = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Name LIKE '%Alice%'"
        )
        
        assert result_lower['success'] is True
        assert result_upper['success'] is True
        assert result_mixed['success'] is True
        
        # 三种大小写应返回相同结果
        names_lower = set(row[0] for row in result_lower['data'] if row[0] != 'Name')
        names_upper = set(row[0] for row in result_upper['data'] if row[0] != 'Name')
        names_mixed = set(row[0] for row in result_mixed['data'] if row[0] != 'Name')
        
        assert names_lower == names_upper == names_mixed
        assert 'Alice' in names_lower

    # ==================== COUNT(DISTINCT)测试 ====================

    def test_count_distinct_no_group_by(self, test_excel_file):
        """测试COUNT(DISTINCT)不带GROUP BY"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT COUNT(DISTINCT Department) FROM Employees"
        )
        
        assert result['success'] is True
        # 应返回不同部门的数量
        data_values = [row[0] for row in result['data'] if row[0] != 'count_distinct_Department']
        assert len(data_values) == 1
        assert isinstance(data_values[0], int)
        assert data_values[0] >= 1

    def test_count_distinct_with_group_by(self, test_excel_file):
        """测试COUNT(DISTINCT)带GROUP BY"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Department, COUNT(DISTINCT Name) FROM Employees GROUP BY Department"
        )
        
        assert result['success'] is True
        data_rows = [row for row in result['data'] if row[0] != 'Department']
        assert len(data_rows) >= 1
        for row in data_rows:
            assert isinstance(row[1], int)

    def test_not_in(self, test_excel_file):
        """测试NOT IN排除列表值"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        # IN匹配指定部门
        result_in = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Department IN ('Sales', 'HR')"
        )
        # NOT IN排除指定部门
        result_not_in = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT Name FROM Employees WHERE Department NOT IN ('Sales', 'HR')"
        )
        
        assert result_in['success'] is True
        assert result_not_in['success'] is True
        
        in_names = set(row[0] for row in result_in['data'] if row[0] != 'Name')
        not_in_names = set(row[0] for row in result_not_in['data'] if row[0] != 'Name')
        
        # 两集合不应有交集
        assert len(in_names & not_in_names) == 0
        # NOT IN结果应只有IT部门的人
        assert not_in_names == {'Bob', 'Eve'}

    def test_count_distinct_with_alias(self, test_excel_file):
        """测试COUNT(DISTINCT)带别名"""
        from src.excel_mcp_server_fastmcp.api.advanced_sql_query import execute_advanced_sql_query
        
        result = execute_advanced_sql_query(
            file_path=test_excel_file,
            sql="SELECT COUNT(DISTINCT Department) AS dept_count FROM Employees"
        )
        
        assert result['success'] is True
        assert 'dept_count' in result.get('query_info', {}).get('returned_columns', [])



"""
Excel MCP Server - 工具模块
"""

from .validators import *
from .parsers import *
from .exceptions import *

__all__ = [
    'ExcelValidator',
    'RangeParser',
    'ExcelException',
    'FileNotFoundError',
    'InvalidFormatError',
    'InvalidRangeError',
    'SheetNotFoundError'
]



"""
Excel MCP Server - 解析工具

提供范围表达式等解析功能
"""

import re
from typing import Dict, Any

from ..models.types import RangeInfo, RangeType
from .exceptions import InvalidRangeError


class RangeParser:
    """范围解析器"""

    @classmethod
    def parse_range_expression(cls, range_expr: str) -> RangeInfo:
        """
        解析范围表达式 (如 'Sheet1!A1:C10' 或 'A1:C10' 或 '1:1' 或 'A:A')

        Args:
            range_expr: 范围表达式

        Returns:
            RangeInfo对象

        Raises:
            InvalidRangeError: 无效的范围表达式
        """
        if not range_expr or not range_expr.strip():
            raise InvalidRangeError("范围表达式不能为空")

        range_expr = range_expr.strip()

        # 分离工作表名和单元格范围
        if '!' in range_expr:
            sheet_name, cell_range = range_expr.split('!', 1)
        else:
            sheet_name = None
            cell_range = range_expr

        # 检测范围类型
        range_type = cls._detect_range_type(cell_range)

        # 规范化范围表达式
        normalized_range = cls._normalize_range(cell_range, range_type)

        return RangeInfo(
            sheet_name=sheet_name,
            cell_range=normalized_range,
            range_type=range_type
        )

    @classmethod
    def _detect_range_type(cls, cell_range: str) -> RangeType:
        """
        检测范围类型

        Args:
            cell_range: 单元格范围字符串

        Returns:
            范围类型

        Raises:
            InvalidRangeError: 无效的范围表达式
        """
        # 检测整行模式 (如 "1:1", "3:5")
        if re.match(r'^\d+:\d+$', cell_range):
            return RangeType.ROW_RANGE

        # 检测整列模式 (如 "A:A", "B:D")
        elif re.match(r'^[A-Z]+:[A-Z]+$', cell_range):
            return RangeType.COLUMN_RANGE

        # 检测单行模式 (如 "1", 只读取第1行)
        elif re.match(r'^\d+$', cell_range):
            return RangeType.SINGLE_ROW

        # 检测单列模式 (如 "A", 只读取A列)
        elif re.match(r'^[A-Z]+$', cell_range):
            return RangeType.SINGLE_COLUMN

        # 检测单元格范围模式 (如 "A1:B2", "A1")
        elif re.match(r'^[A-Z]+\d+$|^[A-Z]+\d+:[A-Z]+\d+$', cell_range):
            return RangeType.CELL_RANGE

        # 如果都不匹配，抛出异常
        else:
            raise InvalidRangeError(f"无效的范围表达式: {cell_range}")

    @classmethod
    def _normalize_range(cls, cell_range: str, range_type: RangeType) -> str:
        """
        规范化范围表达式

        Args:
            cell_range: 原始范围表达式
            range_type: 范围类型

        Returns:
            规范化的范围表达式
        """
        if range_type == RangeType.SINGLE_ROW:
            return f"{cell_range}:{cell_range}"
        elif range_type == RangeType.SINGLE_COLUMN:
            return f"{cell_range}:{cell_range}"
        else:
            return cell_range

    @classmethod
    def validate_range_syntax(cls, range_expr: str) -> bool:
        """
        验证范围表达式语法

        Args:
            range_expr: 范围表达式

        Returns:
            是否为有效语法
        """
        try:
            cls.parse_range_expression(range_expr)
            return True
        except InvalidRangeError:
            return False



# -*- coding: utf-8 -*-
"""
Excel Operations 错误处理和边界情况测试套件

覆盖 excel_operations.py 中的错误处理路径
"""

import pytest
from pathlib import Path
from openpyxl import Workbook

from src.excel_mcp_server_fastmcp.api.excel_operations import ExcelOperations


class TestExcelOperationsErrors:
    """ExcelOperations 错误处理测试"""

    @pytest.fixture
    def valid_test_file(self, temp_dir):
        """创建有效测试文件"""
        file_path = temp_dir / "valid_test.xlsx"
        
        wb = Workbook()
        ws = wb.active
        ws.title = "Sheet1"
        
        ws['A1'] = "ID"
        ws['B1'] = "Name"
        ws['A2'] = 1
        ws['B2'] = "Test"
        
        wb.save(file_path)
        return str(file_path)

    # ==================== 文件不存在错误 ====================

    def test_check_duplicate_ids_file_not_found(self):
        """测试文件不存在时的错误处理"""
        result = ExcelOperations.check_duplicate_ids(
            file_path="/nonexistent/file.xlsx",
            sheet_name="Sheet1",
            id_column=1
        )
        
        assert result['success'] is False
        assert '不存在' in result.get('message', '')

    def test_compare_files_file_not_found(self):
        """测试比较不存在的文件"""
        result = ExcelOperations.compare_files(
            file1_path="/nonexistent/file1.xlsx",
            file2_path="/nonexistent/file2.xlsx"
        )
        
        assert result['success'] is False

    # ==================== 工作表不存在错误 ====================

    def test_check_duplicate_ids_sheet_not_found(self, valid_test_file):
        """测试工作表不存在时的错误处理"""
        result = ExcelOperations.check_duplicate_ids(
            file_path=valid_test_file,
            sheet_name="NonExistentSheet",
            id_column=1
        )
        
        assert result['success'] is False
        assert '工作表不存在' in result.get('message', '')

    # ==================== 空参数测试 ====================

    def test_check_duplicate_ids_empty_file_path(self):
        """测试空文件路径"""
        result = ExcelOperations.check_duplicate_ids(
            file_path="",
            sheet_name="Sheet1",
            id_column=1
        )
        
        assert result['success'] is False

    def test_check_duplicate_ids_empty_sheet_name(self, valid_test_file):
        """测试空工作表名"""
        result = ExcelOperations.check_duplicate_ids(
            file_path=valid_test_file,
            sheet_name="",
            id_column=1
        )
        
        assert result['success'] is False

    # ==================== find_last_row 边界测试 ====================

    def test_find_last_row_empty_sheet(self, valid_test_file):
        """测试空工作表"""
        # 创建空工作表
        wb = Workbook()
        ws = wb.create_sheet("EmptySheet")
        file_path = valid_test_file
        wb.save(file_path)
        
        result = ExcelOperations.find_last_row(
            file_path=file_path,
            sheet_name="EmptySheet"
        )
        
        assert result['success'] is True

    def test_find_last_row_with_column(self, valid_test_file):
        """测试指定列查找最后一行"""
        result = ExcelOperations.find_last_row(
            file_path=valid_test_file,
            sheet_name="Sheet1",
            column="A"
        )
        
        assert result['success'] is True

    def test_find_last_row_with_column_index(self, valid_test_file):
        """测试指定列索引查找最后一行"""
        result = ExcelOperations.find_last_row(
            file_path=valid_test_file,
            sheet_name="Sheet1",
            column=1
        )
        
        assert result['success'] is True

    def test_find_last_row_nonexistent_sheet(self, valid_test_file):
        """测试不存在的工作表"""
        result = ExcelOperations.find_last_row(
            file_path=valid_test_file,
            sheet_name="NonExistentSheet"
        )
        
        assert result['success'] is False


class TestExcelOperationsAdvanced:
    """ExcelOperations 高级功能测试"""

    @pytest.fixture
    def multi_sheet_file(self, temp_dir):
        """创建多工作表测试文件"""
        file_path = temp_dir / "multi_sheet.xlsx"
        
        wb = Workbook()
        
        # 第一个工作表
        ws1 = wb.active
        ws1.title = "Data"
        ws1['A1'] = "ID"
        ws1['A2'] = 1
        ws1['A3'] = 2
        ws1['A4'] = 3
        
        # 第二个工作表
        ws2 = wb.create_sheet("Config")
        ws2['A1'] = "Key"
        ws2['A2'] = "Value"
        
        wb.save(file_path)
        return str(file_path)

    def test_list_sheets(self, multi_sheet_file):
        """测试列出所有工作表"""
        result = ExcelOperations.list_sheets(multi_sheet_file)
        
        assert result['success'] is True
        assert 'Data' in result['sheets']
        assert 'Config' in result['sheets']

    def test_get_file_info(self, multi_sheet_file):
        """测试获取文件信息"""
        result = ExcelOperations.get_file_info(multi_sheet_file)
        
        assert result['success'] is True
        assert 'sheet_count' in result['data']

    def test_get_headers_with_dual_header(self, temp_dir):
        """测试获取双行表头"""
        file_path = temp_dir / "dual_header.xlsx"
        
        wb = Workbook()
        ws = wb.active
        ws.title = "Sheet1"
        
        # 设置双行表头
        ws['A1'] = "描述"
        ws['A2'] = "字段"
        ws['B1'] = "描述2"
        ws['B2'] = "字段2"
        
        wb.save(file_path)
        
        result = ExcelOperations.get_headers(
            file_path=str(file_path),
            sheet_name="Sheet1",
            header_row=1
        )
        
        assert result['success'] is True


class TestExcelOperationsSearch:
    """ExcelOperations 搜索功能测试"""

    @pytest.fixture
    def search_file(self, temp_dir):
        """创建搜索测试文件"""
        file_path = temp_dir / "search_file.xlsx"
        
        wb = Workbook()
        ws = wb.active
        ws.title = "SearchSheet"
        
        # 添加测试数据
        ws['A1'] = "Header"
        ws['A2'] = "Apple"
        ws['A3'] = "Banana"
        ws['A4'] = "APPLE"
        ws['A5'] = "Cherry"
        
        # 添加数字
        ws['B1'] = "Number"
        ws['B2'] = 100
        ws['B3'] = 200
        ws['B4'] = 300
        
        wb.save(file_path)
        return str(file_path)

    def test_search_case_insensitive(self, search_file):
        """测试不区分大小写搜索"""
        result = ExcelOperations.search(
            file_path=search_file,
            pattern="apple",
            sheet_name="SearchSheet",
            case_sensitive=False
        )
        
        assert result['success'] is True

    def test_search_with_column_filter(self, search_file):
        """测试带列过滤的搜索"""
        result = ExcelOperations.search(
            file_path=search_file,
            pattern="Apple",
            sheet_name="SearchSheet"
        )
        
        assert result['success'] is True

    def test_search_range(self, search_file):
        """测试范围搜索"""
        result = ExcelOperations.search(
            file_path=search_file,
            pattern="100",
            sheet_name="SearchSheet",
            range="B1:B5"
        )
        
        assert result['success'] is True


class TestExcelOperationsRange:
    """ExcelOperations 范围操作测试"""

    @pytest.fixture
    def range_file(self, temp_dir):
        """创建范围测试文件"""
        file_path = temp_dir / "range_file.xlsx"
        
        wb = Workbook()
        ws = wb.active
        ws.title = "RangeSheet"
        
        # 添加测试数据
        for i in range(1, 11):
            ws.cell(row=i, column=1, value=i)
            ws.cell(row=i, column=2, value=f"Data{i}")
        
        wb.save(file_path)
        return str(file_path)

    def test_get_range_partial(self, range_file):
        """测试部分范围读取"""
        result = ExcelOperations.get_range(
            file_path=range_file,
            range_expression="RangeSheet!A1:B5"
        )
        
        assert result['success'] is True
        assert len(result['data']) == 5

    def test_get_range_single_row(self, range_file):
        """测试单行范围"""
        result = ExcelOperations.get_range(
            file_path=range_file,
            range_expression="RangeSheet!1:1"
        )
        
        assert result['success'] is True

    def test_get_range_single_column(self, range_file):
        """测试单列范围"""
        result = ExcelOperations.get_range(
            file_path=range_file,
            range_expression="RangeSheet!A:A"
        )
        
        assert result['success'] is True


class TestExcelOperationsEdgeCases:
    """ExcelOperations 边界情况测试"""

    @pytest.fixture
    def edge_file(self, temp_dir):
        """创建边界测试文件"""
        file_path = temp_dir / "edge_file.xlsx"
        
        wb = Workbook()
        ws = wb.active
        ws.title = "EdgeSheet"
        
        # 包含特殊值
        ws['A1'] = ""
        ws['A2'] = None
        ws['A3'] = 0
        ws['A4'] = False
        ws['A5'] = "0"
        
        wb.save(file_path)
        return str(file_path)

    def test_search_empty_cell(self, edge_file):
        """测试搜索空单元格"""
        result = ExcelOperations.search(
            file_path=edge_file,
            pattern="",
            sheet_name="EdgeSheet"
        )
        
        assert result['success'] is True

    def test_search_none_value(self, edge_file):
        """测试搜索None值"""
        result = ExcelOperations.search(
            file_path=edge_file,
            pattern="None",
            sheet_name="EdgeSheet"
        )
        
        assert result['success'] is True

    def test_search_zero_value(self, edge_file):
        """测试搜索0值"""
        result = ExcelOperations.search(
            file_path=edge_file,
            pattern="0",
            sheet_name="EdgeSheet"
        )
        
        assert result['success'] is True



"""
安全功能测试套件

测试所有安全改进功能，确保数据安全和操作可靠性
"""

import pytest
import tempfile
import os
from unittest.mock import patch, MagicMock
from datetime import datetime

from src.excel_mcp_server_fastmcp.server import (
    OperationLogger,
    excel_preview_operation,
    excel_assess_data_impact,
    excel_create_backup,
    excel_restore_backup,
    excel_list_backups,
    excel_get_operation_history,
    _analyze_current_data,
    _assess_operation_risk,
    _generate_safety_recommendations,
    _predict_operation_result
)
from src.excel_mcp_server_fastmcp.utils.validators import ExcelValidator, DataValidationError
from src.excel_mcp_server_fastmcp.api.excel_operations import ExcelOperations


class TestOperationLogger:
    """测试操作日志记录器"""

    def setup_method(self):
        """每个测试方法前的设置"""
        self.logger = OperationLogger()
        self.test_file = "test_operations.xlsx"

    def test_start_session(self):
        """测试开始操作会话"""
        self.logger.start_session(self.test_file)

        assert self.logger.current_session is not None
        assert len(self.logger.current_session) == 1
        assert self.logger.current_session[0]['file_path'] == self.test_file
        assert 'session_id' in self.logger.current_session[0]
        assert 'operations' in self.logger.current_session[0]

    def test_log_operation(self):
        """测试记录操作"""
        self.logger.start_session(self.test_file)

        operation_details = {
            "range": "Sheet1!A1:C10",
            "data_rows": 10
        }

        self.logger.log_operation("update_range", operation_details)

        operations = self.logger.current_session[0]['operations']
        assert len(operations) == 1
        assert operations[0]['operation'] == "update_range"
        assert operations[0]['details'] == operation_details
        assert 'timestamp' in operations[0]

    def test_get_recent_operations(self):
        """测试获取最近操作"""
        self.logger.start_session(self.test_file)

        # 记录多个操作
        for i in range(5):
            self.logger.log_operation(f"operation_{i}", {"step": i})

        recent_ops = self.logger.get_recent_operations(3)
        assert len(recent_ops) == 3
        assert recent_ops[-1]['operation'] == "operation_4"

    def test_get_recent_operations_empty_session(self):
        """测试空会话时获取操作"""
        recent_ops = self.logger.get_recent_operations()
        assert recent_ops == []


class TestEnhancedRangeValidation:
    """测试增强的范围验证功能"""

    def test_validate_valid_ranges(self):
        """测试有效范围格式"""
        valid_ranges = [
            "Sheet1!A1:C10",
            "Data!1:100",
            "Report!A:Z",
            "Summary!5",
            "Analysis!B",
            "技能配置表!A1:Z100"
        ]

        for range_expr in valid_ranges:
            result = ExcelValidator.validate_range_expression(range_expr)
            assert result['success'] is True
            assert 'sheet_name' in result
            assert 'range_info' in result
            assert 'normalized_range' in result

    def test_validate_invalid_ranges(self):
        """测试无效范围格式"""
        invalid_ranges = [
            "A1:C10",  # 缺少工作表名
            "",        # 空字符串
            None,      # None值
            "Sheet1!", # 缺少范围部分
            "Sheet1!INVALID_FORMAT"
        ]

        for range_expr in invalid_ranges:
            with pytest.raises(DataValidationError):
                ExcelValidator.validate_range_expression(range_expr)

    def test_operation_scale_validation(self):
        """测试操作规模验证"""
        # 低风险
        range_info = {'type': 'cell_range', 'start_col': 1, 'end_col': 5, 'start_row': 1, 'end_row': 10}
        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['risk_level'] == "LOW"

        # 中等风险
        range_info = {'type': 'cell_range', 'start_col': 1, 'end_col': 20, 'start_row': 1, 'end_row': 100}
        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['risk_level'] == "MEDIUM"

        # 高风险
        range_info = {'type': 'cell_range', 'start_col': 1, 'end_col': 50, 'start_row': 1, 'end_row': 300}
        result = ExcelValidator.validate_operation_scale(range_info)
        assert result['risk_level'] == "HIGH"

    def test_scale_limits_enforcement(self):
        """测试规模限制强制执行"""
        # 超过行数限制
        range_info = {'type': 'cell_range', 'start_col': 1, 'end_col': 10, 'start_row': 1, 'end_row': 2000}
        with pytest.raises(DataValidationError):
            ExcelValidator.validate_operation_scale(range_info)

        # 超过列数限制
        range_info = {'type': 'cell_range', 'start_col': 1, 'end_col': 200, 'start_row': 1, 'end_row': 10}
        with pytest.raises(DataValidationError):
            ExcelValidator.validate_operation_scale(range_info)


class TestDataAnalysis:
    """测试数据分析功能"""

    def test_analyze_empty_data(self):
        """测试分析空数据"""
        result = _analyze_current_data([])

        assert result['row_count'] == 0
        assert result['column_count'] == 0
        assert result['non_empty_cell_count'] == 0
        assert result['completeness_rate'] == 0.0

    def test_analyze_numeric_data(self):
        """测试分析数值数据"""
        data = [
            [1, 2, 3],
            [4, 5, 6],
            [7, 8, 9]
        ]

        result = _analyze_current_data(data)

        assert result['row_count'] == 3
        assert result['column_count'] == 3
        assert result['non_empty_cell_count'] == 9
        assert result['has_numeric_data'] is True
        assert result['has_text_data'] is False
        assert result['completeness_rate'] == 100.0
        assert result['data_types']['numeric'] == 9

    def test_analyze_mixed_data(self):
        """测试分析混合类型数据"""
        data = [
            [1, "Text", "=SUM(A1)"],
            [None, "", "Another Text"],
            [3.14, "中文", 42]
        ]

        result = _analyze_current_data(data)

        assert result['row_count'] == 3
        assert result['column_count'] == 3
        assert result['non_empty_cell_count'] == 7  # 1, Text, =SUM, Another Text, 3.14, 中文, 42
        assert result['has_numeric_data'] is True
        assert result['has_text_data'] is True
        assert result['has_formulas'] is True
        assert result['data_types']['numeric'] == 3  # 1, 3.14, 42
        assert result['data_types']['text'] == 3  # Text, Another Text, 中文
        assert result['data_types']['formulas'] == 1  # =SUM

    def test_analyze_data_completeness(self):
        """测试数据完整性分析"""
        # 部分填充的数据
        data = [
            [1, None, 3],
            [None, 5, None],
            [7, None, 9]
        ]

        result = _analyze_current_data(data)

        assert result['total_cells'] == 9
        assert result['non_empty_cell_count'] == 5
        assert result['empty_cell_count'] == 4
        assert result['completeness_rate'] == (5/9) * 100


class TestRiskAssessment:
    """测试风险评估功能"""

    def test_assess_low_risk_operation(self):
        """测试低风险操作评估"""
        data_analysis = {
            'non_empty_cell_count': 0,
            'has_formulas': False,
            'completeness_rate': 0.0
        }
        scale_info = {'total_cells': 10}

        result = _assess_operation_risk("update", data_analysis, scale_info)

        assert result['overall_risk'] == "LOW"
        assert result['risk_score'] < 30
        assert result['requires_backup'] is False
        assert result['requires_confirmation'] is False

    def test_assess_high_risk_operation(self):
        """测试高风险操作评估"""
        data_analysis = {
            'non_empty_cell_count': 100,
            'has_formulas': True,
            'completeness_rate': 90.0
        }
        scale_info = {'total_cells': 15000}

        result = _assess_operation_risk("delete", data_analysis, scale_info)

        assert result['overall_risk'] == "HIGH"
        assert result['risk_score'] >= 60
        assert result['requires_backup'] is True
        assert result['requires_confirmation'] is True
        assert "删除操作不可逆" in result['risk_factors']
        assert "包含公式数据" in result['risk_factors']

    def test_safety_recommendations_generation(self):
        """测试安全建议生成"""
        data_analysis = {
            'has_formulas': True,
            'completeness_rate': 85.0,
            'non_empty_cell_count': 50
        }
        risk_assessment = {
            'requires_backup': True,
            'requires_confirmation': True,
            'overall_risk': "HIGH"
        }
        scale_info = {'total_cells': 5000}

        recommendations = _generate_safety_recommendations(
            "update", data_analysis, risk_assessment, scale_info
        )

        assert len(recommendations) > 0
        assert any("建议在操作前创建备份" in rec for rec in recommendations)
        assert any("高风险操作" in rec for rec in recommendations)
        assert any("公式数据" in rec for rec in recommendations)

    def test_operation_result_prediction(self):
        """测试操作结果预测"""
        current_data = [[1, 2], [3, 4]]
        new_data = [[5, 6], [7, 8]]
        scale_info = {'total_cells': 4}

        result = _predict_operation_result("update", current_data, new_data, scale_info)

        assert result['affected_cells'] == 4
        assert result['data_overwrite_count'] == 4
        assert result['data_insert_count'] == 4
        assert result['estimated_time'] == "minimal"


class TestBackupSystem:
    """测试备份系统功能"""

    def setup_method(self):
        """每个测试方法前的设置"""
        self.temp_dir = tempfile.mkdtemp()
        self.test_file = os.path.join(self.temp_dir, "test_data.xlsx")

    def teardown_method(self):
        """每个测试方法后的清理"""
        import shutil
        shutil.rmtree(self.temp_dir, ignore_errors=True)

    def test_create_backup_success(self):
        """测试成功创建备份"""
        # 创建测试文件
        with open(self.test_file, 'w') as f:
            f.write("test content")

        result = excel_create_backup(self.test_file)

        assert result['success'] is True
        assert 'backup_file' in result
        assert 'timestamp' in result
        assert os.path.exists(result['backup_file'])
        assert result['file_size']['original'] == result['file_size']['backup']

    def test_create_backup_nonexistent_file(self):
        """测试创建不存在文件的备份"""
        result = excel_create_backup("nonexistent.xlsx")

        assert result['success'] is False
        assert result['error'] == 'FILE_NOT_FOUND'
        assert "不存在" in result['message']

    def test_list_backups(self):
        """测试列出备份文件"""
        import time

        # 创建测试文件
        with open(self.test_file, 'w') as f:
            f.write("test content")

        # 创建多个备份，确保时间戳不同
        excel_create_backup(self.test_file)
        time.sleep(0.1)  # 确保时间戳不同
        excel_create_backup(self.test_file)

        result = excel_list_backups(self.test_file)

        assert result['success'] is True
        assert result['total_backups'] >= 1  # 至少有一个备份
        assert len(result['backups']) >= 1
        assert all('filename' in backup for backup in result['backups'])

    def test_list_backups_no_backups(self):
        """测试列出不存在的备份"""
        result = excel_list_backups(self.test_file)

        assert result['success'] is True
        assert result.get('total_backups', 0) == 0
        assert result.get('backups', []) == []


class TestIntegrationSecurityWorkflow:
    """测试完整的安全工作流程集成"""

    def setup_method(self):
        """每个测试方法前的设置"""
        self.temp_dir = tempfile.mkdtemp()
        self.test_file = os.path.join(self.temp_dir, "security_test.xlsx")

    def teardown_method(self):
        """每个测试方法后的清理"""
        import shutil
        shutil.rmtree(self.temp_dir, ignore_errors=True)

    @patch('src.excel_mcp_server_fastmcp.server.ExcelOperations.get_range')
    def test_complete_safe_update_workflow(self, mock_get_range):
        """测试完整的安全更新工作流程"""
        # 模拟现有数据
        mock_get_range.return_value = {
            'success': True,
            'data': [[1, 2, 3], [4, 5, 6]]
        }

        # 1. 数据影响评估
        assessment_result = excel_assess_data_impact(
            self.test_file,
            "Sheet1!A1:C3",
            "update",
            [[7, 8, 9], [10, 11, 12]]
        )

        assert assessment_result['success'] is True
        assert 'risk_assessment' in assessment_result
        assert 'safety_recommendations' in assessment_result

        # 2. 操作预览
        preview_result = excel_preview_operation(
            self.test_file,
            "Sheet1!A1:C3",
            "update"
        )

        assert preview_result['success'] is True
        assert 'impact_assessment' in preview_result
        assert 'safety_warning' in preview_result

        # 3. 验证风险评估一致性
        risk_level = assessment_result['risk_assessment']['overall_risk']
        assert risk_level in ["LOW", "MEDIUM", "HIGH"]

    def test_validation_prevents_dangerous_operations(self):
        """测试验证机制阻止危险操作"""
        # 测试无效范围格式 - 应该返回错误而不是抛出异常
        result = excel_assess_data_impact(
            self.test_file,
            "invalid_range",  # 无效格式
            "update"
        )
        assert result['success'] is False
        assert result['error'] == 'VALIDATION_FAILED'

        # 测试范围验证 - 这个会抛出异常
        with pytest.raises(DataValidationError):
            ExcelValidator.validate_range_expression("A1:C10")  # 缺少工作表名

    @patch('src.excel_mcp_server_fastmcp.server.ExcelOperations.get_range')
    def test_high_risk_operation_detection(self, mock_get_range):
        """测试高风险操作检测"""
        # 模拟大量数据
        large_data = [[f"cell_{i}_{j}" for j in range(100)] for i in range(100)]
        mock_get_range.return_value = {
            'success': True,
            'data': large_data
        }

        # 大范围操作应该被识别为高风险
        assessment_result = excel_assess_data_impact(
            self.test_file,
            "Sheet1!A1:CV100",  # 100x100 = 10000 cells
            "delete"
        )

        assert assessment_result['success'] is True
        assert assessment_result['risk_assessment']['overall_risk'] in ["HIGH", "MEDIUM"]
        assert assessment_result['risk_assessment']['requires_backup'] is True

    def test_operation_history_tracking(self):
        """测试操作历史跟踪"""
        # 创建临时文件用于测试
        with open(self.test_file, 'w') as f:
            f.write("test")

        # 获取操作历史
        history_result = excel_get_operation_history(self.test_file, 10)

        assert history_result['success'] is True
        assert 'operations' in history_result
        assert 'statistics' in history_result

        # 验证统计信息结构
        stats = history_result['statistics']
        assert 'total_operations' in stats
        assert 'operation_types' in stats
        assert 'success_rate' in stats


class TestSecurityPerformance:
    """测试安全功能的性能"""

    def test_large_scale_validation_performance(self):
        """测试大规模验证的性能"""
        import time

        # 测试各种范围格式的验证性能
        test_ranges = [
            "Sheet1!A1:Z1000",
            "Data!1:10000",
            "Report!A:ZZ",
            "Analysis!1:100000"
        ]

        start_time = time.time()

        for range_expr in test_ranges:
            try:
                result = ExcelValidator.validate_range_expression(range_expr)
                if result['success']:
                    scale_info = ExcelValidator.validate_operation_scale(result['range_info'])
            except DataValidationError:
                pass  # 某些范围可能无效，这是正常的

        end_time = time.time()

        # 验证性能要求（所有验证应在1秒内完成）
        assert (end_time - start_time) < 1.0

    def test_data_analysis_performance(self):
        """测试数据分析的性能"""
        import time

        # 创建大型数据集
        large_data = [[f"data_{i}_{j}" for j in range(50)] for i in range(100)]

        start_time = time.time()
        result = _analyze_current_data(large_data)
        end_time = time.time()

        # 验证分析结果正确性
        assert result['row_count'] == 100
        assert result['column_count'] == 50
        assert result['non_empty_cell_count'] == 5000

        # 验证性能要求（大型数据分析应在0.5秒内完成）
        assert (end_time - start_time) < 0.5


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""
Excel MCP Server - Excel管理模块

提供Excel文件和工作表管理功能
"""

import logging
from typing import List, Optional
from pathlib import Path
from openpyxl import Workbook, load_workbook
from openpyxl.utils import get_column_letter

from ..models.types import SheetInfo, OperationResult
from ..utils.validators import ExcelValidator
from ..utils.exceptions import SheetNotFoundError, DataValidationError

# 设置日志级别为INFO以便调试
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class ExcelManager:
    """Excel文件和工作表管理器"""

    @classmethod
    def create_file(
        cls,
        file_path: str,
        sheet_names: Optional[List[str]] = None
    ) -> OperationResult:
        """
        创建新的Excel文件

        Args:
            file_path: 要创建的Excel文件路径
            sheet_names: 工作表名称列表

        Returns:
            OperationResult: 创建操作的结果
        """
        try:
            # 验证路径，默认允许覆盖已存在的文件
            validated_path = ExcelValidator.validate_file_for_creation(file_path, overwrite=True)

            # 创建工作簿
            workbook = Workbook()

            # 处理工作表
            if sheet_names:
                # 删除默认工作表
                default_sheet = workbook.active
                workbook.remove(default_sheet)

                # 创建指定的工作表
                created_sheets = []
                for i, sheet_name in enumerate(sheet_names):
                    if not sheet_name or not sheet_name.strip():
                        raise DataValidationError(f"工作表名称不能为空: 索引 {i}")

                    sheet = workbook.create_sheet(title=sheet_name.strip())
                    created_sheets.append(SheetInfo(
                        index=i,
                        name=sheet.title,
                        max_row=1,
                        max_column=1,
                        max_column_letter='A'
                    ))

                # 设置第一个工作表为活动工作表
                if created_sheets:
                    workbook.active = workbook[created_sheets[0].name]
            else:
                # 使用默认工作表
                created_sheets = [SheetInfo(
                    index=0,
                    name='Sheet1',
                    max_row=1,
                    max_column=1,
                    max_column_letter='A'
                )]

            # 确保目录存在
            Path(validated_path).parent.mkdir(parents=True, exist_ok=True)

            # 保存文件
            workbook.save(validated_path)

            return OperationResult(
                success=True,
                data=created_sheets,
                message=f"成功创建Excel文件，包含{len(created_sheets)}个工作表",
                metadata={
                    'file_path': validated_path,
                    'total_sheets': len(created_sheets)
                }
            )

        except Exception as e:
            logger.error(f"创建Excel文件失败: {e}")
            return OperationResult(
                success=False,
                error=str(e)
            )

    def __init__(self, file_path: str):
        """
        初始化Excel管理器

        Args:
            file_path: Excel文件路径
        """
        self.file_path = ExcelValidator.validate_file_path(file_path)

    def list_sheets(self) -> OperationResult:
        """
        列出所有工作表

        Returns:
            OperationResult: 包含工作表信息列表的结果
        """
        from .excel_reader import ExcelReader
        reader = ExcelReader(self.file_path)
        return reader.list_sheets()

    def create_sheet(
        self,
        sheet_name: str,
        index: Optional[int] = None
    ) -> OperationResult:
        """
        在Excel文件中创建新工作表，支持中文字符

        Args:
            sheet_name: 新工作表名称
            index: 插入位置索引

        Returns:
            OperationResult: 创建操作的结果
        """
        try:
            # 验证工作表名称
            if not sheet_name or not sheet_name.strip():
                raise DataValidationError("工作表名称不能为空")

            # 规范化工作表名称，处理中文字符
            sheet_name = self._normalize_sheet_name(sheet_name.strip())

            # 加载Excel文件
            workbook = load_workbook(self.file_path)

            # 检查工作表名称是否已存在
            if sheet_name in workbook.sheetnames:
                raise DataValidationError(f"工作表名称已存在: {sheet_name}")

            # 验证索引
            total_sheets = len(workbook.sheetnames)
            if index is not None:
                if index < 0 or index > total_sheets:
                    raise DataValidationError(f"索引超出范围: {index}，应在 0-{total_sheets} 之间")

            # 创建新工作表，使用安全的编码处理
            try:
                new_sheet = workbook.create_sheet(title=sheet_name, index=index)
            except Exception as sheet_error:
                # 如果直接创建失败，尝试使用ASCII兼容的名称
                logger.warning(f"创建工作表失败，尝试备用方法: {sheet_error}")
                fallback_name = self._create_fallback_name(sheet_name, workbook.sheetnames)
                new_sheet = workbook.create_sheet(title=fallback_name, index=index)
                logger.info(f"使用备用名称创建工作表: {fallback_name}")
                sheet_name = fallback_name

            # 保存文件
            workbook.save(self.file_path)

            # 获取新工作表信息
            sheet_info = SheetInfo(
                index=workbook.sheetnames.index(sheet_name),
                name=new_sheet.title,
                max_row=1,
                max_column=1,
                max_column_letter='A'
            )

            return OperationResult(
                success=True,
                data=sheet_info,
                message=f"成功创建工作表: {sheet_name}",
                metadata={
                    'file_path': self.file_path,
                    'total_sheets': len(workbook.sheetnames),
                    'all_sheets': workbook.sheetnames
                }
            )

        except Exception as e:
            logger.error(f"创建工作表失败: {e}")
            return OperationResult(
                success=False,
                error=str(e)
            )

    def _normalize_sheet_name(self, name: str) -> str:
        """
        规范化工作表名称，确保与Excel兼容

        Args:
            name: 原始工作表名称

        Returns:
            str: 规范化后的名称
        """
        import re

        # Excel工作表名称限制：
        # 1. 不能超过31个字符
        # 2. 不能包含: / \ ? * [ ] :
        # 3. 不能以单引号开头或结尾

        # 移除或替换无效字符
        invalid_chars = r'[/\\?*\[\]:]'
        name = re.sub(invalid_chars, '_', name)

        # 移除首尾的单引号和空格
        name = name.strip("' \t\n\r")

        # 限制长度（考虑中文字符占用更多字节）
        if len(name) > 31:
            # 对于中文字符，保守截取到25个字符
            name = name[:25] + "..."

        # 确保名称不为空
        if not name:
            name = "Sheet"

        return name

    def _create_fallback_name(self, original_name: str, existing_names: list) -> str:
        """
        创建备用工作表名称

        Args:
            original_name: 原始名称
            existing_names: 已存在的名称列表

        Returns:
            str: 备用名称
        """
        import re

        # 尝试创建ASCII兼容的名称
        fallback_base = "Sheet"

        # 尝试从原始名称中提取英文字符
        ascii_chars = re.findall(r'[a-zA-Z0-9]', original_name)
        if ascii_chars:
            fallback_base = ''.join(ascii_chars)[:10]  # 最多取10个字符

        # 确保名称唯一
        counter = 1
        fallback_name = fallback_base
        while fallback_name in existing_names:
            fallback_name = f"{fallback_base}_{counter}"
            counter += 1

        return fallback_name

    def delete_sheet(self, sheet_name: str) -> OperationResult:
        """
        删除Excel文件中的工作表

        Args:
            sheet_name: 要删除的工作表名称

        Returns:
            OperationResult: 删除操作的结果
        """
        try:
            # 验证工作表名称
            if not sheet_name or not sheet_name.strip():
                raise DataValidationError("工作表名称不能为空")

            sheet_name = sheet_name.strip()

            # 加载Excel文件
            workbook = load_workbook(self.file_path)

            # 检查工作表是否存在
            if sheet_name not in workbook.sheetnames:
                raise SheetNotFoundError(f"工作表不存在: {sheet_name}")

            # 检查是否为最后一个工作表
            if len(workbook.sheetnames) <= 1:
                raise DataValidationError("无法删除最后一个工作表，Excel文件至少需要一个工作表")

            # 记录删除前的信息
            deleted_sheet_index = workbook.sheetnames.index(sheet_name)
            deleted_sheet_name = sheet_name  # 保存要删除的工作表名称

            logger.info(f"准备删除工作表: {deleted_sheet_name}, 索引: {deleted_sheet_index}")
            logger.info(f"删除前工作表列表: {workbook.sheetnames}")

            # 删除工作表
            workbook.remove(workbook[sheet_name])

            logger.info(f"删除后工作表列表: {workbook.sheetnames}")

            # 如果删除的是第一个工作表，设置新的第一个为活动工作表
            if deleted_sheet_index == 0 and workbook.sheetnames:
                workbook.active = 0
            elif deleted_sheet_index < len(workbook.sheetnames):
                workbook.active = deleted_sheet_index
            else:
                workbook.active = deleted_sheet_index - 1

            # 保存文件
            workbook.save(self.file_path)

            # 返回更新后的工作表信息
            remaining_sheet_infos = []
            for i, sheet_name_iter in enumerate(workbook.sheetnames):
                sheet = workbook[sheet_name_iter]
                remaining_sheet_infos.append(SheetInfo(
                    index=i,
                    name=sheet_name_iter,
                    max_row=sheet.max_row,
                    max_column=sheet.max_column,
                    max_column_letter=get_column_letter(sheet.max_column)
                ))

            return OperationResult(
                success=True,
                data=remaining_sheet_infos,  # 添加data字段
                message=f"成功删除工作表: {deleted_sheet_name}",
                metadata={
                    'file_path': self.file_path,
                    'deleted_sheet': deleted_sheet_name,
                    'deleted_index': deleted_sheet_index,
                    'new_active_sheet': workbook.active.title,
                    'remaining_sheets': workbook.sheetnames,
                    'total_sheets': len(workbook.sheetnames)
                }
            )

        except Exception as e:
            logger.error(f"删除工作表失败: {e}")
            return OperationResult(
                success=False,
                error=str(e)
            )

    def rename_sheet(self, old_name: str, new_name: str) -> OperationResult:
        """
        重命名Excel文件中的工作表

        Args:
            old_name: 原工作表名称
            new_name: 新工作表名称

        Returns:
            OperationResult: 重命名操作的结果
        """
        try:
            # 验证工作表名称
            if not old_name or not old_name.strip():
                raise DataValidationError("原工作表名称不能为空")
            if not new_name or not new_name.strip():
                raise DataValidationError("新工作表名称不能为空")

            old_name = old_name.strip()
            new_name = new_name.strip()

            if old_name == new_name:
                raise DataValidationError("新名称与原名称相同，无需重命名")

            # 加载Excel文件
            workbook = load_workbook(self.file_path)

            # 检查原工作表是否存在
            if old_name not in workbook.sheetnames:
                raise SheetNotFoundError(f"原工作表不存在: {old_name}")

            # 检查新名称是否已存在
            if new_name in workbook.sheetnames:
                raise DataValidationError(f"新工作表名称已存在: {new_name}")

            # 获取工作表
            sheet = workbook[old_name]
            old_index = workbook.sheetnames.index(old_name)

            # 重命名工作表
            sheet.title = new_name

            # 保存文件
            workbook.save(self.file_path)

            # 构建重命名后的工作表信息
            renamed_sheet_info = SheetInfo(
                index=old_index,
                name=new_name,
                max_row=sheet.max_row,
                max_column=sheet.max_column,
                max_column_letter=get_column_letter(sheet.max_column) if sheet.max_column > 0 else 'A'
            )

            return OperationResult(
                success=True,
                message=f"成功将工作表 '{old_name}' 重命名为 '{new_name}'",
                data=renamed_sheet_info,
                metadata={
                    'file_path': self.file_path,
                    'old_name': old_name,
                    'new_name': new_name,
                    'sheet_index': old_index,
                    'all_sheets': workbook.sheetnames
                }
            )

        except Exception as e:
            logger.error(f"重命名工作表失败: {e}")
            return OperationResult(
                success=False,
                error=str(e)
            )

    @staticmethod
    def get_file_info(file_path: str) -> OperationResult:
        """
        获取Excel文件的详细信息

        Args:
            file_path: Excel文件路径

        Returns:
            OperationResult: 文件信息结果
        """
        try:
            import os
            from datetime import datetime

            if not os.path.exists(file_path):
                raise FileNotFoundError(f"文件不存在: {file_path}")

            # 获取文件系统信息
            stat_info = os.stat(file_path)
            file_size = stat_info.st_size
            created_time = datetime.fromtimestamp(stat_info.st_ctime).strftime('%Y-%m-%d %H:%M:%S')
            modified_time = datetime.fromtimestamp(stat_info.st_mtime).strftime('%Y-%m-%d %H:%M:%S')

            # 获取文件格式
            file_format = Path(file_path).suffix.lower().lstrip('.')

            # 加载工作簿获取详细信息
            workbook = load_workbook(file_path, read_only=True)
            sheet_count = len(workbook.sheetnames)
            sheet_names = workbook.sheetnames
            has_macros = file_format == 'xlsm'

            # 简单统计数据行数（使用第一个工作表）
            total_rows = 0
            total_cols = 0
            if workbook.worksheets:
                ws = workbook.worksheets[0]  # 使用第一个工作表
                if ws.max_row and ws.max_column:
                    total_rows = ws.max_row
                    total_cols = ws.max_column

            workbook.close()

            return OperationResult(
                success=True,
                message=f"成功获取文件信息: {file_path}",
                data={
                    'file_path': file_path,
                    'file_name': Path(file_path).name,
                    'file_size': file_size,
                    'file_size_mb': round(file_size / 1024 / 1024, 2),
                    'created_time': created_time,
                    'modified_time': modified_time,
                    'format': file_format,
                    'sheet_count': sheet_count,
                    'sheet_names': sheet_names,
                    'has_macros': has_macros,
                    'total_rows': total_rows,
                    'total_cols': total_cols
                },
                metadata={
                    'file_format': file_format,
                    'is_macro_enabled': has_macros,
                    'sheet_summary': {name: f"工作表{i+1}" for i, name in enumerate(sheet_names)}
                }
            )

        except Exception as e:
            logger.error(f"获取文件信息失败: {e}")
            return OperationResult(
                success=False,
                error=f"获取文件信息失败: {str(e)}"
            )



"""
新API的专门测试用例
测试拆分后的API功能是否正常工作
"""
import pytest
import sys
import os
from pathlib import Path

# 获取项目根目录并添加到路径
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))
sys.path.insert(0, str(project_root / "src"))

# 直接导入src.excel_mcp_server_fastmcp.server模块
import src.excel_mcp_server_fastmcp.server as server


class TestNewAPIs:
    """测试拆分后的新API"""

    @pytest.fixture
    def sample_file(self):
        """使用现有的测试文件"""
        return "data/test-api-comprehensive.xlsx"

    def test_excel_list_sheets_new(self, sample_file):
        """测试新的excel_list_sheets - 只返回工作表列表"""
        result = server.excel_list_sheets(sample_file)

        assert result['success'] is True
        assert 'sheets' in result
        assert 'total_sheets' in result
        # active_sheet概念已被移除
        assert 'active_sheet' not in result
        # 确保不再返回sheets_with_headers
        assert 'sheets_with_headers' not in result

        # 检查返回的工作表列表
        sheets = result['sheets']
        assert isinstance(sheets, list)
        assert len(sheets) > 0

    def test_excel_get_sheet_headers_new(self, sample_file):
        """测试新的excel_get_sheet_headers - 专门获取表头"""
        result = server.excel_get_sheet_headers(sample_file)

        assert result['success'] is True
        assert 'sheets_with_headers' in result
        assert 'total_sheets' in result
        # 确保返回表头信息
        assert 'file_path' in result

        # 检查表头信息结构
        sheets_with_headers = result['sheets_with_headers']
        assert isinstance(sheets_with_headers, list)

        for sheet_info in sheets_with_headers:
            assert 'name' in sheet_info
            assert 'header_count' in sheet_info
            # headers字段在空工作表时可能被清理，使用默认值处理
            headers = sheet_info.get('headers', [])
            assert isinstance(headers, list)
            assert isinstance(sheet_info['header_count'], int)

    def test_api_separation_completeness(self, sample_file):
        """测试API拆分的完整性 - 新老API功能等价"""
        # 新API的分离调用
        sheets_result = server.excel_list_sheets(sample_file)
        headers_result = server.excel_get_sheet_headers(sample_file)

        # 验证功能完整性
        assert sheets_result['success'] is True
        assert headers_result['success'] is True

        # 工作表列表应该一致
        sheets_from_list = set(sheets_result['sheets'])
        sheets_from_headers = set([s['name'] for s in headers_result['sheets_with_headers']])
        assert sheets_from_list == sheets_from_headers

    def test_excel_format_cells_custom(self):
        """测试自定义格式化API"""
        # 注意：这是一个示例，实际测试需要真实的Excel文件
        # 这里主要测试参数验证和函数调用
        test_formatting = {
            'font': {'bold': True, 'color': 'FF0000'},
            'fill': {'color': 'FFFF00'}
        }

        # 测试参数结构（不实际运行，避免文件依赖）
        try:
            # 这应该抛出文件不存在的错误，但参数结构是正确的
            server.excel_format_cells("nonexistent.xlsx", "Sheet1", "A1:B1", formatting=test_formatting)
        except Exception as e:
            # 期望的是文件相关错误，不是参数错误
            assert "nonexistent.xlsx" in str(e) or "No such file" in str(e) or "does not exist" in str(e)

    def test_excel_format_cells(self):
        """测试预设格式化API"""
        valid_presets = ["title", "header", "data", "highlight", "currency"]

        for preset in valid_presets:
            try:
                # 测试预设验证（不实际运行，避免文件依赖）
                server.excel_format_cells("nonexistent.xlsx", "Sheet1", "A1:B1", preset=preset)
            except Exception as e:
                # 期望的是文件相关错误，不是预设验证错误
                assert "nonexistent.xlsx" in str(e) or "No such file" in str(e) or "does not exist" in str(e)

    def test_invalid_preset(self):
        """测试无效预设的错误处理"""
        try:
            result = server.excel_format_cells("any.xlsx", "Sheet1", "A1:B1", preset="invalid_preset")
            # 如果返回结果而不是抛出异常，检查错误信息
            if 'success' in result:
                assert result['success'] is False
                assert "未知的预设样式" in result.get('message', '')
        except Exception:
            # 如果抛出异常也是可接受的
            pass

    def test_parameter_clarity(self):
        """测试新API的参数清晰度"""
        import inspect

        # 检查excel_list_sheets参数
        sig_list = inspect.signature(server.excel_list_sheets)
        params_list = list(sig_list.parameters.keys())
        assert params_list == ['file_path'], f"excel_list_sheets参数应该只有file_path，实际为: {params_list}"

        # 检查excel_get_sheet_headers参数
        sig_headers = inspect.signature(server.excel_get_sheet_headers)
        params_headers = list(sig_headers.parameters.keys())
        assert params_headers == ['file_path'], f"excel_get_sheet_headers参数应该只有file_path，实际为: {params_headers}"

        # 跳过检查已合并的excel_format_cells_custom，因为它已经合并到excel_format_cells中

        # 检查excel_format_cells参数
        sig_preset = inspect.signature(server.excel_format_cells)
        params_preset = list(sig_preset.parameters.keys())
        expected_preset = ['file_path', 'sheet_name', 'range', 'formatting', 'preset']
        assert params_preset == expected_preset, f"excel_format_cells参数不符合预期: {params_preset}"

    def test_single_responsibility_principle(self, sample_file):
        """测试单一职责原则的实现"""
        # excel_list_sheets应该只返回工作表相关信息，不包含表头
        list_result = server.excel_list_sheets(sample_file)
        assert 'sheets' in list_result
        assert 'sheets_with_headers' not in list_result

        # excel_get_sheet_headers应该专注于表头信息
        headers_result = server.excel_get_sheet_headers(sample_file)
        assert 'sheets_with_headers' in headers_result
        # 可以包含基础的sheets信息用于上下文，但主要功能是表头

        # 两个API返回的数据结构应该不同，体现不同职责
        assert set(list_result.keys()) != set(headers_result.keys())


if __name__ == "__main__":
    # 运行测试
    pytest.main([__file__, "-v"])



"""
Excel MCP Server - Excel读取模块

提供Excel文件读取功能
使用python-calamine（Rust引擎）加速纯数据读取，openpyxl作为格式化读取的后备方案
"""

import logging
from typing import List, Dict, Any, Optional
from openpyxl import load_workbook
from openpyxl.utils import range_boundaries, column_index_from_string, get_column_letter

from ..models.types import (
    RangeInfo, RangeType, CellInfo, SheetInfo,
    ExcelData, ExcelDimensions, OperationResult
)
from ..utils.validators import ExcelValidator
from ..utils.parsers import RangeParser
from ..utils.exceptions import SheetNotFoundError

logger = logging.getLogger(__name__)

# 尝试导入calamine，不可用时降级到openpyxl
try:
    from python_calamine import CalamineWorkbook
    _HAS_CALAMINE = True
except ImportError:
    _HAS_CALAMINE = False
    logger.debug("python-calamine未安装，读取性能将受影响")


class ExcelReader:
    """Excel文件读取器（calamine加速 + openpyxl后备）"""

    def __init__(self, file_path: str):
        """
        初始化Excel读取器

        Args:
            file_path: Excel文件路径
        """
        self.file_path = ExcelValidator.validate_file_path(file_path)
        self._workbook_cache = {}  # 缓存不同参数的openpyxl工作簿
        self._calamine_wb = None   # calamine工作簿缓存（只缓存一个，因为不支持参数变体）

    def _get_calamine_workbook(self):
        """
        获取缓存的calamine工作簿或加载新工作簿

        Returns:
            CalamineWorkbook: calamine工作簿对象
        """
        if self._calamine_wb is None and _HAS_CALAMINE:
            self._calamine_wb = CalamineWorkbook.from_path(self.file_path)
        return self._calamine_wb

    def _get_workbook(self, read_only: bool = True, data_only: bool = False):
        """
        获取缓存的工作簿或加载新工作簿（openpyxl）

        Args:
            read_only: 是否以只读模式打开
            data_only: 是否只读取值（不包含公式）

        Returns:
            Workbook: openpyxl工作簿对象
        """
        cache_key = (read_only, data_only)
        if cache_key not in self._workbook_cache:
            self._workbook_cache[cache_key] = load_workbook(
                self.file_path,
                read_only=read_only,
                data_only=data_only
            )
        return self._workbook_cache[cache_key]

    def close(self):
        """关闭所有缓存的工作簿"""
        for workbook in self._workbook_cache.values():
            if workbook is not None:
                workbook.close()
        self._workbook_cache.clear()
        self._calamine_wb = None

    def list_sheets(self) -> OperationResult:
        """
        获取Excel文件中所有工作表的信息

        Returns:
            OperationResult: 包含所有工作表信息的结果
        """
        # calamine快速路径
        if _HAS_CALAMINE:
            try:
                wb = self._get_calamine_workbook()
                sheets_info = []
                for i, name in enumerate(wb.sheet_names):
                    ws = wb.get_sheet_by_name(name)
                    sheet_info = SheetInfo(
                        index=i,
                        name=name,
                        max_row=ws.height,
                        max_column=ws.width,
                        max_column_letter=get_column_letter(ws.width)
                    )
                    sheets_info.append(sheet_info)

                return OperationResult(
                    success=True,
                    data=sheets_info,
                    metadata={
                        'file_path': self.file_path,
                        'total_sheets': len(sheets_info)
                    }
                )
            except Exception as e:
                logger.debug(f"calamine list_sheets失败，回退openpyxl: {e}")

        # openpyxl后备路径
        try:
            workbook = self._get_workbook(read_only=True)

            sheets_info = []
            for i, sheet_name in enumerate(workbook.sheetnames):
                sheet = workbook[sheet_name]

                sheet_info = SheetInfo(
                    index=i,
                    name=sheet_name,
                    max_row=sheet.max_row,
                    max_column=sheet.max_column,
                    max_column_letter=get_column_letter(sheet.max_column)
                )
                sheets_info.append(sheet_info)

            return OperationResult(
                success=True,
                data=sheets_info,
                metadata={
                    'file_path': self.file_path,
                    'total_sheets': len(sheets_info)
                }
            )

        except Exception as e:
            logger.error(f"获取工作表列表失败: {e}")
            return OperationResult(
                success=False,
                error=str(e)
            )

    def get_range(
        self,
        range_expression: str,
        include_formatting: bool = False
    ) -> OperationResult:
        """
        获取Excel文件中指定范围的数据

        Args:
            range_expression: 范围表达式
            include_formatting: 是否包含格式信息

        Returns:
            OperationResult: 包含范围数据的结果
        """
        # 解析范围表达式
        range_info = RangeParser.parse_range_expression(range_expression)

        # calamine快速路径（仅限纯数据读取）
        if not include_formatting and _HAS_CALAMINE:
            try:
                return self._get_range_calamine(range_info, range_expression)
            except Exception as e:
                logger.debug(f"calamine get_range失败，回退openpyxl: {e}")

        # openpyxl路径（格式化读取或calamine不可用）
        return self._get_range_openpyxl(range_info, range_expression, include_formatting)

    def _get_range_calamine(self, range_info: RangeInfo, range_expression: str) -> OperationResult:
        """使用calamine快速读取范围数据（纯值，无格式）"""
        wb = self._get_calamine_workbook()
        sheet_name = range_info.sheet_name

        if not sheet_name or not sheet_name.strip():
            raise SheetNotFoundError("工作表名称不能为空，必须明确指定工作表")

        if sheet_name not in wb.sheet_names:
            raise SheetNotFoundError(
                f"工作表不存在: {sheet_name}，可用工作表: {', '.join(wb.sheet_names)}"
            )

        ws = wb.get_sheet_by_name(sheet_name)
        all_rows = list(ws.to_python())

        data, dimensions = self._extract_range_from_rows(
            all_rows, range_info
        )

        return OperationResult(
            success=True,
            data=data,
            metadata={
                'file_path': self.file_path,
                'range': range_expression,
                'range_type': range_info.range_type.value,
                'sheet_name': sheet_name,
                'dimensions': dimensions.__dict__
            }
        )

    def _extract_range_from_rows(
        self,
        all_rows: List[List],
        range_info: RangeInfo
    ) -> tuple[ExcelData, ExcelDimensions]:
        """从已加载的行数据中提取指定范围（0-based索引）"""
        data = []

        if range_info.range_type in [RangeType.ROW_RANGE, RangeType.SINGLE_ROW]:
            row_parts = range_info.cell_range.split(':')
            start_row = int(row_parts[0]) - 1  # 转为0-based
            end_row = int(row_parts[1]) - 1

            for r in range(start_row, end_row + 1):
                row_values = all_rows[r] if r < len(all_rows) else []
                row_data = []
                for c, val in enumerate(row_values):
                    col_letter = get_column_letter(c + 1)
                    cell_info = CellInfo(
                        coordinate=f"{col_letter}{r + 1}",
                        value=self._normalize_calamine_value(val)
                    )
                    row_data.append(cell_info)
                data.append(row_data)

            max_col = max((len(row) for row in data), default=0)
            dimensions = ExcelDimensions(
                rows=end_row - start_row + 1,
                columns=max_col,
                start_row=start_row + 1,
                start_column=1
            )

        elif range_info.range_type in [RangeType.COLUMN_RANGE, RangeType.SINGLE_COLUMN]:
            col_parts = range_info.cell_range.split(':')
            start_col = column_index_from_string(col_parts[0]) - 1  # 转为0-based
            end_col = column_index_from_string(col_parts[1]) - 1

            for r, row_values in enumerate(all_rows):
                row_data = []
                for c in range(start_col, end_col + 1):
                    col_letter = get_column_letter(c + 1)
                    val = row_values[c] if c < len(row_values) else None
                    cell_info = CellInfo(
                        coordinate=f"{col_letter}{r + 1}",
                        value=self._normalize_calamine_value(val)
                    )
                    row_data.append(cell_info)
                data.append(row_data)

            dimensions = ExcelDimensions(
                rows=len(all_rows),
                columns=end_col - start_col + 1,
                start_row=1,
                start_column=start_col + 1
            )

        else:
            # CELL_RANGE
            min_col, min_row, max_col, max_row = range_boundaries(range_info.cell_range)

            for r in range(min_row - 1, max_row):  # 转为0-based
                row_values = all_rows[r] if r < len(all_rows) else []
                row_data = []
                for c in range(min_col - 1, max_col):  # 转为0-based
                    col_letter = get_column_letter(c + 1)
                    val = row_values[c] if c < len(row_values) else None
                    cell_info = CellInfo(
                        coordinate=f"{col_letter}{r + 1}",
                        value=self._normalize_calamine_value(val)
                    )
                    row_data.append(cell_info)
                data.append(row_data)

            dimensions = ExcelDimensions(
                rows=max_row - min_row + 1,
                columns=max_col - min_col + 1,
                start_row=min_row,
                start_column=min_col
            )

        return data, dimensions

    @staticmethod
    def _normalize_calamine_value(val):
        """归一化calamine值：整型浮点数转整数（25.0→25），与openpyxl行为一致"""
        if isinstance(val, float) and val == int(val) and not (val != val):  # 排除NaN
            return int(val)
        return val

    def _get_range_openpyxl(self, range_info: RangeInfo, range_expression: str, include_formatting: bool) -> OperationResult:
        """使用openpyxl读取范围数据（支持格式化）"""
        try:
            workbook = self._get_workbook(read_only=True, data_only=True)
            sheet = self._get_worksheet_openpyxl(workbook, range_info.sheet_name)

            data, dimensions = self._get_range_data_openpyxl(
                sheet, range_info, include_formatting
            )

            return OperationResult(
                success=True,
                data=data,
                metadata={
                    'file_path': self.file_path,
                    'range': range_expression,
                    'range_type': range_info.range_type.value,
                    'sheet_name': sheet.title,
                    'dimensions': dimensions.__dict__
                }
            )

        except Exception as e:
            logger.error(f"获取范围数据失败: {e}")
            return OperationResult(
                success=False,
                error=str(e)
            )

    def _get_worksheet_openpyxl(self, workbook, sheet_name: Optional[str]):
        """获取openpyxl工作表"""
        if not sheet_name or not sheet_name.strip():
            raise SheetNotFoundError(f"工作表名称不能为空，必须明确指定工作表")

        if not workbook.sheetnames:
            raise SheetNotFoundError(f"Excel文件中没有任何工作表")

        if sheet_name not in workbook.sheetnames:
            raise SheetNotFoundError(f"工作表不存在: {sheet_name}，可用工作表: {', '.join(workbook.sheetnames)}")

        return workbook[sheet_name]

    # 向后兼容别名（excel_operations.py直接调用此方法）
    _get_worksheet = _get_worksheet_openpyxl

    def _get_range_data_openpyxl(
        self,
        sheet,
        range_info: RangeInfo,
        include_formatting: bool
    ) -> tuple[ExcelData, ExcelDimensions]:
        """根据范围类型获取数据（openpyxl）"""

        if range_info.range_type in [RangeType.ROW_RANGE, RangeType.SINGLE_ROW]:
            return self._get_row_data(sheet, range_info, include_formatting)
        elif range_info.range_type in [RangeType.COLUMN_RANGE, RangeType.SINGLE_COLUMN]:
            return self._get_column_data(sheet, range_info, include_formatting)
        else:
            return self._get_cell_range_data(sheet, range_info, include_formatting)

    def _get_row_data(
        self,
        sheet,
        range_info: RangeInfo,
        include_formatting: bool
    ) -> tuple[ExcelData, ExcelDimensions]:
        """获取行范围数据"""
        row_parts = range_info.cell_range.split(':')
        start_row = int(row_parts[0])
        end_row = int(row_parts[1])

        # 优化：使用iter_rows而不是手动遍历所有列
        data = []
        max_col_found = 0

        for row in sheet.iter_rows(min_row=start_row, max_row=end_row, values_only=False):
            row_data = []
            last_data_col = 0

            for col_idx, cell in enumerate(row, 1):
                if cell.value is not None:
                    last_data_col = col_idx

            if last_data_col == 0:
                last_data_col = 1

            for col_idx in range(1, last_data_col + 1):
                if col_idx - 1 < len(row):
                    cell = row[col_idx - 1]
                else:
                    cell = sheet.cell(row=start_row + len(data), column=col_idx)
                cell_info = self._create_cell_info(cell, include_formatting)
                row_data.append(cell_info)

            data.append(row_data)
            max_col_found = max(max_col_found, last_data_col)

        dimensions = ExcelDimensions(
            rows=end_row - start_row + 1,
            columns=max_col_found,
            start_row=start_row,
            start_column=1
        )

        return data, dimensions

    def _get_column_data(
        self,
        sheet,
        range_info: RangeInfo,
        include_formatting: bool
    ) -> tuple[ExcelData, ExcelDimensions]:
        """获取列范围数据"""
        col_parts = range_info.cell_range.split(':')
        start_col = column_index_from_string(col_parts[0])
        end_col = column_index_from_string(col_parts[1])
        max_row = sheet.max_row

        data = []
        for row_idx in range(1, max_row + 1):
            row_data = []
            for col_idx in range(start_col, end_col + 1):
                cell = sheet.cell(row=row_idx, column=col_idx)
                cell_info = self._create_cell_info(cell, include_formatting)
                row_data.append(cell_info)
            data.append(row_data)

        dimensions = ExcelDimensions(
            rows=max_row,
            columns=end_col - start_col + 1,
            start_row=1,
            start_column=start_col
        )

        return data, dimensions

    def _get_cell_range_data(
        self,
        sheet,
        range_info: RangeInfo,
        include_formatting: bool
    ) -> tuple[ExcelData, ExcelDimensions]:
        """获取单元格范围数据"""
        min_col, min_row, max_col, max_row = range_boundaries(range_info.cell_range)

        data = []
        for row_idx in range(min_row, max_row + 1):
            row_data = []
            for col_idx in range(min_col, max_col + 1):
                cell = sheet.cell(row=row_idx, column=col_idx)
                cell_info = self._create_cell_info(cell, include_formatting)
                row_data.append(cell_info)
            data.append(row_data)

        dimensions = ExcelDimensions(
            rows=max_row - min_row + 1,
            columns=max_col - min_col + 1,
            start_row=min_row,
            start_column=min_col
        )

        return data, dimensions

    def _create_cell_info(self, cell, include_formatting: bool) -> CellInfo:
        """创建单元格信息对象"""
        if hasattr(cell, 'coordinate'):
            coordinate = cell.coordinate
        else:
            coordinate = f"{get_column_letter(cell.column)}{cell.row}" if hasattr(cell, 'row') and hasattr(cell, 'column') else "A1"

        cell_info = CellInfo(
            coordinate=coordinate,
            value=cell.value
        )

        if include_formatting and hasattr(cell, 'data_type'):
            cell_info.data_type = cell.data_type
            cell_info.number_format = getattr(cell, 'number_format', None)
            cell_info.font = str(cell.font) if hasattr(cell, 'font') and cell.font else None
            cell_info.fill = str(cell.fill) if hasattr(cell, 'fill') and cell.fill else None

        return cell_info
